<?php
  /**
  *
  */
  class TSubmitButton extends TControl {
    /** @event(TComponent $sender) Button has been clicked */
    public $onClick = false;            
    private $clicked = false;
    public $newRow = false;
    /** @var string Text of the confirmation dialog? */
    public $confirmText = null;    
    /** @var bool Use as default submit button? */
    #public $default = false;
    protected function _getDefault()  {
      if($this->form) {
        return ($this->form->defaultButton == $this);
      }
    }
    protected function _setDefault($value)  {
      if($this->form) {
        if($value) {
          $this->form->defaultButton = $this;
        } else {
          if($this->form->defaultButton == $this) {
            $this->form->defaultButton = null;
          }
        }
      }
    }    
    /**
    * Set this button as an default to be used when Enter key is pressed
    *
    * @return void
    */
    public function setAsDefault()  {
      if($this->form) {
        $this->form->defaultButton = $this;
      }
    }
    /**
    * Has the button been clicked
    *
    * @return bool
    */
    public function isClicked()  {
      return $this->clicked;    
    }
    /**
    * Called when the button has been clicked
    *
    * @return void
    */
    public function click() {
      $this->clicked = true;
      $this->fireEvent('onClick');
      return true;    
    }
    public function loadInput(&$input) {

    }
    public function validate() {
      return true;
    }
    public function __construct(TComponent $parent, $name = null, $properties = null) {
      parent::__construct($parent,$name,$properties);
      $this->actions['click'] = ACTION_AFTER_INPUT;
    }      
  }

  /**
  *
  */
  abstract class TFormatedControl extends TControl {
    /** @var TCustomValue Custom format*/
    public $format = null;
    public function valueChanged(TCustomValue $value) {}
    /**
    * Returns $this->format or field->value
    *
    * @return TCustomValue
    */
    public function getFormat() {
      if($this->format) {
        return $this->format;
      }
      return false;
    }
    public function getText() {
      return null;
    }
  }

  class TFieldLinkedControl extends TFormatedControl {
    /** @var string Field name */
    public $field = null;
    
    public function getCaption( ) {
      if($this->caption === null) {
        if($this->field !== null) {      
//        if($f = $this->getField()) {
          return $this->field;
        } else {
          return $this->name;
        }
      } else {
        return $this->caption;
      }
    }
    public function getText() {
      if($f = $this->getField()) {
        return $f->getAsDisplay();
      } else {
        return null;
      }
    }    
        
    /**
    * Return dataset field specified by the $field property 
    *
    * @return TField
    */
    public function getField() {    
      if(($this->dataset === null) || ($this->field === null)) {
        return false;
      }
      return $this->getDataset()->getField($this->field);    
    }
    /**
    * Returns $this->format or field->value
    *
    * @return TCustomValue
    */
    public function getFormat() {
      if($this->format) {
        return $this->format;
      }
      if($field = $this->getField()) {
        return $field->value;
      }
      return false;
    }      
  }
  
  /**
  *
  */
  class TInputControl extends TFieldLinkedControl {
    /** @var string Value */    
    public $value = null;
    /** @var bool Is the field required */    
    public $required = null;
    /** @var bool Should the field have unique value? */
    public $unique = null;
    /** @var string Simple WHERE query */
    public $uniqueScope = null;
    /** @var int Trim the input value */
    public $trim = true;
    /** @var int Save hash instead of plain text */
    protected $hash = HASH_NONE;
        
    private $valueLock = false;
    
    
    public function getText() {
      return $value;
    }
            
    public function valueChanged(TCustomValue $value) {
      if(!$this->valueLock) {
        $this->value = $value->getAsInput();
      }
    }
        
    /**
    * Is the field required (checks $required and fields $notNull) 
    *
    * @return bool
    */
    public function isRequired( )  {
      if($this->required === null) {
        if(/** @var TField */ $field = $this->getField()) {
          return ($field->notNull == true);
        }
      } else {
        return $this->required;
      }
    }

    /**
    * Should the field be unique? (checks $unique and fields $unique) 
    *
    * @return bool
    */
    public function isUnique() {
      if($this->unique === null) {
        if(/** @var TField */ $field = $this->getField()) {
          return $field->unique;
        }
      } else {
        return $this->unique;
      }
    }
        
    /**
    * Returns true if no value has been submited
    *
    * @return bool
    */
    public function isEmpty() {
      return (($this->value == '') || ($this->value === null));
    }

    public function loadFromDataset()  {
      $this->valueLock = true;
      if(/** @var TField */ $field = $this->getField()) {
        if($this->format) {

          $this->format->setRaw($field->value->getRaw());        
          $this->value = $this->format->getAsInput();
        } else {        
          $this->value = $field->value->getAsInput();
        }
      }

      $this->valueLock = false;      
    }
    public function saveToDataset()  {
      return $this->validate();      
    }

    private function inbuiltValidation() {
      if($this->isRequired() && $this->isEmpty()) {
        $this->setError(E_VALUE_REQUIRED);
        return false;
      }

      /** @var TField */ $field = $this->getField();
      if($this->format) {
        /** @var TCustomValue */
        $format = $this->format;
      } elseif($field) {
        $format = $field->value;
      } else {
        return true;
      }

      if($this->trim) {
        $value = trim($this->value);
      } else {
        $value = $this->value;
      }

      if($this->isEmpty()) {
        $format->clear();
        return true;
      } else {
        $verbose = $format->verbose; 
        $strict = $format->strict;

        $format->verbose = false;
        $format->strict = false;
        if($this->hash != HASH_NONE) {
          switch($this->hash) {
            case HASH_SHA1:
              $value = sha1($value);
              break;
            default:
              break;  
          }
        }                         
        if($format->setAsInput($value)) {

          $res = true;
          if($this->format && $field) {
            $field->value->strict = false;
            $field->value->verbose = false;
            if(!$field->value->setRaw($format->getRaw())) {            
              $this->setError($field->value->getLastError(),$field->value->getLastErrorNo());
              $res = false;
            }
            $field->value->strict = true;
            $field->value->verbose = true;
          }
        } else {
          $this->setError($format->getLastError(),$format->getLastErrorNo());
          $res = false;
        }
        $format->verbose = $verbose;
        $format->strict = $strict;
        return $res;
      }
    }    
    /**
    * Check if the values submited by user are valid
    *
    * @return bool
    */
    public function validate() {
      $this->valueLock = true;
      if($this->inbuiltValidation()) {
        $valid = $this->fireEvent('onValidate');
        $this->valueLock = false;
        $res = (($valid === null) || $valid);
                
        if($res && $this->isUnique()) {
          /** @var TField */
          $field = $this->getField();
          /** @var TDataSet */
          $dataset = $field->dataset;
          /** @var TCustomConnection */
          $db = $dataset->connection;
          
          if(/** @var TDataSet */ $x = $db->query('SELECT COUNT(*) FROM '.$field->orgTable
            .' WHERE '.$field->orgName.' = '.$field->value->getAsSqlQuoted()
            .' AND NOT ('.$dataset->getPrimaryKeyCondition().')'
            .($this->uniqueScope !== null ? ' AND '.$this->uniqueScope : '')
          )) {
            if($x->fetchValue(0) > 0) {
              $this->setError(E_DUPLICATE_VALUE);
              $res = false;
            }
          }
        }
      } else {
        $this->valueLock = false;
        $res = false;
      }
      return $res;      
    }
    public function allSaved() {
      $this->loadFromDataset();
    }
    public function loadInput(&$input) {
      if(isset($input[$this->name])) {
        $this->value = (string)$input[$this->name];          
      }
    }
  }

  class TAutoCompleteOptions extends TObject {
    /** @var bool Auto complete enabled */
    public $enabled = false;
    /** @var int Min length of the input to start searching */
    public $minLength = 2;
    /** @var int Delay in microseconds to wait before search starts */
    public $delay = 300;
    /** @var int Max rows to be returned via dynamic ajax query */
    public $rowCount = 0;    
    public function toJson() {
      $obj = new stdClass();
      $obj->minLength = $this->minLength;
      $obj->delay = $this->delay;      
      return $obj;
    }
  }
  
  /**
  *
  */
  class TDatePickerOptions extends TObject {
    /** @var bool Auto complete enabled */
    public $enabled = true;
    /** @var bool Display year selector */
    public $yearSelector = true;
    /** @var bool Display month selector*/
    public $monthSelector = true;
    /** @var bool Display days from other months */
    public $otherMonthDaysVisible = true;
    /** @var bool Allow click on days from other months */
    public $otherMonthDaysEnabled = true;
    /** @var bool Display Now and Close button  */
    public $displayButtons = true;
    /**
    * Converts strftime format to date picker format 
    *
    * @param string
    * @return string
    */
    public function convertFormat($format)  {
      $format = str_replace(
        array('%e','%d','%j','%a','%A','%m','%b','%B','%y','%Y','%D'),
        array('d' ,'dd','oo','D' ,'DD','mm','M' ,'MM','y' ,'yy','mm/dd/y'),
        $format
      );
      $format = preg_replace('/( *\%[^\%] *)/',"'$1'",$format);
      return formatDateTime($format);
    }    
    public function toJson()  {
      $obj = $this->getLocaleSettings();
      $obj->changeYear = $this->yearSelector;
      $obj->changeMonth = $this->monthSelector;
      $obj->showOtherMonths = $this->otherMonthDaysVisible;
      $obj->selectOtherMonths = $this->otherMonthDaysEnabled;
      return $obj;
    }        
    protected function getLocaleSettings() {
      $obj = new stdClass();
      $obj->closeText = LNG_DATEPICKER_CLOSE;
  		$obj->prevText = LNG_DATEPICKER_EARLIER;
  		$obj->nextText = LNG_DATEPICKER_LATER;
  		$obj->currentText = LNG_DATEPICKER_NOW;

  		$obj->monthNames = array();
  		$obj->monthNamesShort  = array();
  		$obj->dayNames = array();
  		$obj->dayNamesShort = array();
  		$obj->dayNamesMin  = array();
  		
  		
  		for($i = 1; $i < 13; $i++) {
        $date = strtotime('2000-'.$i.'-01');
        $obj->monthNames[] = formatDateTime('%B',$date);
        $obj->monthNamesShort[] = formatDateTime('%b',$date);
      }
      $date = strtotime('LAST SUNDAY');
      $pondeli = $date;
  		for($i = 1; $i < 8; $i++) {
        $obj->dayNames[] = formatDateTime('%A',$date);
        $obj->dayNamesShort[] = formatDateTime('%a',$date);          		
        $date = strtotime('+1 DAY',$date);
      }
      $obj->dayNamesMin = $obj->dayNamesShort;
              		
  		$obj->weekHeader = LNG_DATEPICKER_TYDEN;
  		$obj->dateFormat = 'dd.mm.yy';
  		$obj->firstDay = LNG_WEEK_START;
  		$obj->isRTL = false;
  		$obj->showMonthAfterYear = false;
  		$obj->yearSuffix = '';
      return $obj;    
    }
  }  
  /**
  *              
  */
  class TEditBox extends TInputControl {
    /** @var bool If True and no width is specified, it will be adjusted to fit maxLength */
    public $autoSize = true;
    /** @var TAutoCompleteOptions */
    public $autoComplete;
    /** @var string[] List of values for auto completion */
    public $autoCompleteList = array();
    /** @var TDatePickerOptions */
    public $datePicker;
    /** @event(TComponent $sender, string $filter) Fetch autocomplete list */
    public $onFetchList = false;
    /** @var bool Replace input text with asterisks */
    public $passwordInput = false;
    /** @var bool Keep password value filled in */
    public $keepPassword = false;
    /** @var bool Allow browser triggered autocomplete */
    public $browserAutoComplete = true;
    /** @var bool Show caption as input text if empty */
    public $notBlank = false;
    public $hash = HASH_NONE;
    public function toJson($flags = null) {
      if($flags == 'autoComplete') {        
        if(/** @var TCustomValue */ $format = $this->getFormat()) {
          $list = array();        
          $value = $format->getRaw();
          $s = $format->strict; 
          $format->strict = false;                
          foreach($this->autoCompleteList as $item) {
            if($format) {
              if($format->setRaw($item)) {
                $list[] = $format->getAsInput();
              }
            }
          }
          $format->setRaw($value);          
          $format->strict = $s;
          return $list;
        } else {
          return $this->autoCompleteList;
        }
      }
    }    
    public function __construct(TComponent $parent, $name = null, $properties = null) {
      parent::__construct($parent,$name,$properties);
      $this->autoComplete = new TAutoCompleteOptions();
      $this->datePicker = new TDatePickerOptions();      
      $this->actions['fetchList'] = ACTION_BEFORE_INPUT;
    }
    public function fetchList($filter = null) {
      if($this->fireEvent('onFetchList') === true) {
        exit;
      }
      if(count($this->autoCompleteList) == 0) {
        if(
          (/** @var TField */ $field = $this->getField())
          && (/** @var TCustomConnection */$db = $this->getDb())
          && (/** @var TDataSet */$res = $db->query(
            'SELECT DISTINCT '.$db->quoteFieldName($field->orgName)
            .' FROM '.$db->quoteFieldName($field->orgTable)
            .($filter !== null ?
            ' WHERE '.$db->quoteFieldName($field->orgName)
            ." LIKE '%".$db->escape($filter)."%'"
            : '')
            .' ORDER BY '.$db->quoteFieldName($field->orgName)
            .($this->autoComplete->rowCount > 0 ?            
            ' LIMIT '.$this->autoComplete->rowCount
            : '')
          ))          
        ) {
          while($val = $res->fetchArray()) {
            $this->autoCompleteList[] = $val[0];
          }
        }
      } elseif($filter !== null) {
        foreach(array_keys($this->autoCompleteList) as $key) {
          if(!containsText($filter,$this->autoCompleteList[$key])) {
            unset($this->autoCompleteList[$key]);
          }
        }
      }
      return true;
    } 
  }

  class TTextArea extends TInputControl {
    /** @var int Number of rows */
    public $rows = 5;
    /** @var int Number of columns */
    public $cols = 40;    
  }

  abstract class TCustomSelectOption extends TCollectionItem {
    /** @var TSelectBox */
    public $selectBox;
    /** @var mixed Width of the control */
    public $width = null;
    /** @var mixed Height of the control */
    public $height = null;
    /** @var string Additional custom CSS style */
    public $cssStyle = null;
    /** @var bool Should the control be visible? */
    public $visible = true;
    /** @var string Caption */
    public $caption = null;
    /** @var string Short description shown in the hint window */
    public $title = null;
    /** @var string Additional CSS class */
    public $cssClass = null;
    /** @var bool Is the control enabled? */
    public $enabled = true;
    /**
    * Returns caption to display
    *
    * @return string
    */
    abstract public function getCaption();
    /**
    * Returns index of this option
    *
    * @return int
    */     
    abstract public function getIndex();
    /**
    * Returns unique identifier for this option
    *
    * @return string
    */     
    abstract public function getId();
    public function __construct($parent) {
      parent::__construct($parent);
      $this->selectBox = $parent;
    }
    /** @var string Value of the option or group */
    public $value;
    
    public function toJson($flags = null) {
      $obj = new stdClass();      
      $obj->id = $this->getId();
      $obj->value = $this->value;

//      $obj->optionValue = $this->value; 
      $obj->label = $this->getCaption();
//      $obj->value = $obj->label;      
      $obj->title = $this->title;
      $obj->cssClass = $this->cssClass;
      $obj->cssStyle = $this->cssStyle;            
      return $obj; 
    }        
  }
  class TSelectOptionGroup extends TCustomSelectOption {
    public function getId() {
      return $this->selectBox->getId().'-groups-'.$this->getIndex();
    }
    public function getCaption() {
      if($this->caption === null) {
        return 'Group '.$this->getIndex();
      } else {
        return $this->caption;
      }
    }
    public function getIndex() {
      return array_search($this,$this->selectBox->groups,true); 
    }  
  }
  class TSelectOption extends TCustomSelectOption {
    /**
    * Is this option selected 
    *
    * @return bool
    */
    public function isSelected( )  {
      return $this->selectBox->isOptionSelected($this);
    }
    public function toJson($flags = null) {
      $obj = parent::toJson($flags);
      $obj->selected = $this->isSelected();
      return $obj;
    }
    /**
    * Select this option
    *
    * @return void
    */
    public function select( )  {
      $this->selectBox->selectOption($this);    
    }
    /**
    * Deselect this option
    *
    * @return void
    */
    public function deselect( )  {
      $this->selectBox->deselectOption($this);
    }
    public function getId() {
      return $this->selectBox->getId().'-options-'.$this->getIndex();
    }
    /** @var int Index of a group */
    public $group = null;
    public function getCaption() {
      if($this->caption === null) {
        return $this->value;
      } else {
        return $this->caption;
      }
    }
    public function getIndex() {
      return array_search($this,$this->selectBox->options,true); 
    }
  }
  
  /**
  *
  */
  class TSelectBox extends TInputControl {
    /** @var TSelectOption[] */
    public $options = array();
    /** @var TSelectOptionGoup[] */
    public $groups = array();
    /** @var bool */
    public $showEmptyGroups = false;
    /** @var int Display style */
    public $displayStyle = SELECTBOX_STANDARD;
    /** @var int Number of rows to display */
    public $rows = 1;
    public function __clone( )  {
      $this->cloneProperty('options');
    }
    public function isMultiSelect( )  {
      return false;    
    }
    public function toJson($flags = null) {
      if($flags == 'options') {
        $list = array();
        foreach($this->options as /** @var TCustomSelectOption */ $option) {
          if($option->visible) {
            $list[] = $option->toJson($flags);
          }
        }
        return $list;
      }
    }
    /**
    * Get options in JSON format
    *
    * @return void
    */
    public function filter($term)  {
      foreach($this->options as $key => /** @var TSelectOption */ $option) {
        if(!containsText($term,$option->getCaption())) {
          unset($this->options[$key]);
        }
      }
      return true;
    }
    /**
    * Returns true if the option is selected
    *
    * @param TSelectOption
    * @return bool
    */
    public function isOptionSelected(TSelectOption $option)  {
      return (sameStr($this->value,$option->value));
    }
    /**
    * Selects the option
    *
    * @param TSelectOption
    * @return void
    */
    public function selectOption(TSelectOption $option)  {
      $this->value = $option->value;
    }
    /**
    * Deselects the option
    *
    * @param TSelectOption
    * @return void
    */
    public function deselectOption(TSelectOption $option)  {
      if($this->isOptionSelected($option)) {
        $this->value = null;
      }
    }    
    /**
    * Adds and returns new option 
    *
    * @return TSelectOption
    */
    public function addOption($value = null,$caption = null, $group = null)  {
      $opt = new TSelectOption($this);
      $opt->value = $value;
      $opt->caption = $caption;
      $opt->group = $group;
      $this->options[] = $opt;
      return $opt;      
    }
    /**
    * Adds and returns new option group 
    *
    * @return TSelectOption
    */
    public function addGroup($value = null, $caption = null)  {
      $opt = new TSelectOptionGroup($this);
      $opt->value = $value;
      $opt->caption = $caption;
      $this->groups[] = $opt;
      return $opt;
    }
    public function __construct(TComponent $parent, $name = null, $properties = null) {
      parent::__construct($parent,$name,$properties);
      $this->actions['filter'] = ACTION_BEFORE_INIT;
    }    
  }
  
  class TLookupSelectBox extends TSelectBox {
    /** @var string SELECT id, caption[, title[, group]] FROM... */
    public $query;
    /** @var string SELECT id, caption FROM... */
    public $groupQuery;
    public function filter($term)  {
      $dummyQuery = sqlExplode($this->query);
      $realQuery = $dummyQuery;

      if(isset($dummyQuery['WHERE'])) {
        $dummyQuery['WHERE'] = '('.$dummyQuery['WHERE'].') AND 1 = 0';
      } else {
        $dummyQuery['WHERE'] = '1 = 0';
      }
      /** @var TCustomConnection */
      $db = $this->getDb();

      if(/** @var TDataSet */ $res = $db->query(sqlImplode($dummyQuery))) {
        $res->fetchFields();
        $x = $db->quoteFieldName($res->fieldsByIndex[1]->table)
          .'.'.$db->quoteFieldName($res->fieldsByIndex[1]->name)." LIKE '%".$db->escape($term)."%'";
        if(isset($realQuery['WHERE'])) {
          $realQuery['WHERE'] = '('.$realQuery['WHERE'].') AND '.$x;
        } else {
          $realQuery['WHERE'] = $x;
        }
      }
      $this->query = sqlImplode($realQuery);
      return true;      
    }    
    public function __construct(TComponent $parent, $name = null, $properties = null) {
      parent::__construct($parent,$name,$properties);
      $this->actions['filter'] = ACTION_BEFORE_INIT;
      $this->additionalCssClasses[] = 'rSelectBox';
    }
    public function init() {
      /** @var TCustomConnection */
      $db = $this->getDb();
      if(/** @var TDataSet */ $res = $db->query($this->query)) {
        while($arr = $res->fetchArray()) {
          $opt = $this->addOption($arr[0],$arr[1]); 
          if(isset($arr[2])) {
            $opt->title = $arr[2];
          }
          if(isset($arr[3])) {
            $opt->group = $arr[3];
          }
        }
      }
      if(($this->groupQuery !== null) && (/** @var TDataSet */ $res = $db->query($this->groupQuery))) {
        while($arr = $res->fetchArray()) {
          $this->addGroup($arr[0],$arr[1]);
        }
      }      
    }
  }

  class TGroupBox extends TControl {
    /** @var bool Allow user to hide the group by clicking its header */
    public $expandable = false;
    /** @var bool Should the group be initialy expanded? */
    public $expanded = true;
    public function loadInput(&$input) {
      if(isset($input[$this->getId().'-expanded'])) {
        $this->expanded = ($input[$this->getId().'-expanded'] == '1'); 
      }
      parent::loadInput($input);
    }    
  }
  class TToolBar extends TGroupBox {
    public $displayCaption = false;
  }
  class TSubmitToolBar extends TToolBar {
    public $displayCaption = true;
    public $caption = LNG_BOLD_FIELDS_ARE_REQUIRED;  
    public function __construct(TComponent $parent, $name = null, $properties = null) {
      parent::__construct($parent,$name,$properties);
      $this->additionalCssClasses[] = 'rToolBar';
    }  
  }    
/*  class TToolBar extends TPanel {
    public function __construct(TComponent $parent) {
      parent::__construct($parent);
      $this->additionalCssClasses[] = 'rSubmitButton';
    }  
  }*/
  
  /**
  *
  */
  class TResetButton extends TSubmitButton {
    public function __construct(TComponent $parent, $name = null, $properties = null) {
      parent::__construct($parent,$name,$properties);
      $this->additionalCssClasses[] = 'rSubmitButton';
      $this->actions['click'] = ACTION_BEFORE_INPUT;
    }
    public function click() {
      if($this->fireEvent('onClick') === true) {
        return true;
      }
      $this->form->reset();
      return true;
    }
  }

  class TRelationControl extends TFormatedControl {
    /** @var string Name of the field specifying current row */
    public $fieldThis;
    /** @var string Name of the field specifying connected row */
    public $fieldRelated;
    /** @var bool If true, it doesn't matter if current row value is specified by filedThis or fieldRelated */
    public $relatedRows = false;
    /** @var string Table containing relation rows */
    public $relationTable;
    /** @var string[] Selected values */
    public $values = array();
    /** @var int Min required count of selected items */
    public $minCount = 0;
    /** @var int Max required count of selected items */
    public $maxCount = 0;
    public function isRequired( )  {
      return $this->minCount > 0;
    }    
    public function saveToDataset() {
      return $this->validate();
    }
    public function validate() {
      $cnt = count($this->values);
      if($cnt < $this->minCount) {
        $this->setError(E_TOO_FEW_ITEMS,array($this->minCount));
        return false;
      }
      if(($this->maxCount > 0) & ($cnt > $this->maxCount)) {
        $this->setError(E_TOO_MANY_ITEMS,array($this->maxCount));
        return false;
      }
      return true;
    }    
    public function loadInput(&$input) {
      $id = $this->getId(); 
      if(isset($input[$id])) {
/*        var_dump($input[$id]);
        exit;*/
        if(!is_array($input[$id])) {
          $this->values = array($input[$id]);
        } else {
          $this->values = $input[$id];
        }
      } else {
        $this->values = array();
      }
    }
    public function datasetSaved() {

      /** @var TDataSet */
      $dataset = $this->getDataset();

      if($this->fieldRelated || $this->fieldThis || $this->relationTable) {
        if(!($this->fieldRelated && $this->fieldThis && $this->relationTable)) {
          throw new EException(E_REQUIRED_SETTINGS_MISSING,array('relationTable, fieldThis, fieldRelated'));
        }
      } else {
        return true;
      }
      /** @var TCustomConnection */
      $db = $this->getDb();

      $key = null;
      $keyQuoted = null;
      foreach($dataset->fields as /** @var TField */$field) {
        if($field->primaryKey) {
          $key = $field->value->getAsSql();
          $keyQuoted = $field->value->getAsSqlQuoted();
          $keyOld = $field->getOldValueAsSqlQuoted();                    
          break;
        }      
      }
      if($key === null) {
        throw new EException(E_RECORD_COULD_NOT_BE_IDENTIFICATED);
      }

      if(!$db->execute(
        'DELETE FROM '
        .$db->quoteFieldName($this->relationTable)
        .' WHERE '
        .$db->quoteFieldName($this->fieldThis).' = '.$keyOld
        .($this->relatedRows
          ? ' OR '.$db->quoteFieldName($this->fieldRelated).' = '.$keyOld
          : '')
        )
      ) {
        return false;
      }
      if(count($this->values) == 0) {
        return true;
      }
      $values = $this->values;      
      foreach(array_keys($this->values) as $i) {
        $values[$i] = '('.$keyQuoted.', '.$db->quote($values[$i]).')';
      }
      return $db->execute(
        'INSERT INTO '
        .$db->quoteFieldName($this->relationTable)
        .' ('.$db->quoteFieldName($this->fieldThis).', '.$db->quoteFieldName($this->fieldRelated).')'
        .' VALUES '.implode(',',$values)      
      );
    }
    public function loadFromDataset() {
      /** @var TDataSet */
      $dataset = $this->getDataset();
      if($dataset->isInserting()) {
        return;
      }
      if($this->fieldRelated || $this->fieldThis || $this->relationTable) {
        if(!($this->fieldRelated && $this->fieldThis && $this->relationTable)) {
          throw new EException(E_REQUIRED_SETTINGS_MISSING,array('relationTable, fieldThis, fieldRelated'));
        }
      } else {
        return;
      }
      /** @var TCustomConnection */
      $db = $this->getDb();

      $key = null;
      $keyQuoted = null;
      foreach($dataset->fields as /** @var TField */$field) {
        if($field->primaryKey) {
          $key = $field->oldValue;
          $keyQuoted = $field->getOldValueAsSqlQuoted();          
          break;
        }      
      }
      if($key === null) {
        throw new EException(E_RECORD_COULD_NOT_BE_IDENTIFICATED);
      }
      if(/** @var TDataSet */ $res = $db->query('SELECT '
        .$db->quoteFieldName($this->fieldThis).', '
        .$db->quoteFieldName($this->fieldRelated)
        .' FROM '
        .$db->quoteFieldName($this->relationTable)
        .' WHERE '
        .$db->quoteFieldName($this->fieldThis).' = '.$keyQuoted
        .($this->relatedRows
          ? ' OR '.$db->quoteFieldName($this->fieldRelated).' = '.$keyQuoted
          : ''
        )        
      )) {
        while($row = $res->fetchAssoc()) {
          if($this->relatedRows) {
            if($row[$this->fieldRelated] == $key) {
              $this->values[] = $row[$this->fieldThis];
            } else {
              $this->values[] = $row[$this->fieldRelated];
            }
          } else {
            $this->values[] = $row[$this->fieldRelated];
          }
        }
      }

    }
  }

  class TMultiSelectBox extends TRelationControl {
    /** @var TSelectOption[] */
    public $options = array();
    /** @var TSelectOptionGoup[] */
    public $groups = array();
    /** @var bool */
    public $showEmptyGroups = false;    
    /** @var int Display style */
    public $displayStyle = SELECTBOX_STANDARD;    
    /** @var int Number of rows to display */
    public $rows = 5;
      /** @var string Name of a panel to show when new item should be added */
    public $addItemPanel = null;
    public function isMultiSelect()  {
      return true;    
    }    
    public function toJson($flags = null) {
      if($flags == 'options') {
        $list = array();
        foreach($this->options as /** @var TCustomSelectOption */ $option) {
          if($option->visible) {
            $list[] = $option->toJson($flags);
          }
        }
        return $list;
      }
    }
    /**
    * Filter options
    *
    * @return void
    */
    public function filter($term)  {
      foreach($this->options as $key => /** @var TSelectOption */ $option) {
        if(!containsText($term,$option->getCaption())) {
          unset($this->options[$key]);
        }
      }
      return true;
    }
    /**
    * Returns true if the option is selected
    *
    * @param TSelectOption
    * @return bool
    */
    public function isOptionSelected(TSelectOption $option)  {
      return (array_search($option->value,$this->values) !== false);
    }
    /**
    * Selects the option
    *
    * @param TSelectOption
    * @return void
    */
    public function selectOption(TSelectOption $option)  {
      foreach($this->values as $value) {
        if(sameText($value,$option->value)) {
          return;
        }
      }
      $this->values[] = $option->value;

    }
    /**
    * Deselects the option
    *
    * @param TSelectOption
    * @return void
    */
    public function deselectOption(TSelectOption $option)  {
      foreach($this->values as $key => $value) {
        if(sameText($value,$option->value)) {
          unset($this->values[$key]);
        }
      }
    }    
    /**
    * Adds and returns new option 
    *
    * @return TSelectOption
    */
    public function addOption($value = null,$caption = null, $group = null)  {
      $opt = new TSelectOption($this);
      $opt->value = $value;
      $opt->caption = $caption;
      $opt->group = $group;
      $this->options[] = $opt;
      return $opt;      
    }
    /**
    * Adds and returns new option group 
    *
    * @return TSelectOption
    */
    public function addGroup($value = null, $caption = null)  {
      $opt = new TSelectOptionGroup($this);
      $opt->value = $value;
      $opt->caption = $caption;
      $this->groups[] = $opt;
      return $opt;
    }
    public function __construct(TComponent $parent, $name = null, $properties = null) {
      parent::__construct($parent,$name,$properties);
      $this->actions['filter'] = ACTION_BEFORE_INIT;
      $this->additionalCssClasses[] = 'rSelectBox';
    }    
  }
  
  class TLookupMultiSelectBox extends TMultiSelectBox {
    /** @var string SELECT id, caption[, group] FROM... */
    public $query;
    /** @var string SELECT id, caption FROM... */
    public $groupQuery;
    public function filter($term, $onlySelected = false)  {
      $dummyQuery = sqlExplode($this->query);
      $realQuery = $dummyQuery;

      if(isset($dummyQuery['WHERE'])) {
        $dummyQuery['WHERE'] = '('.$dummyQuery['WHERE'].') AND 1 = 0';
      } else {
        $dummyQuery['WHERE'] = '1 = 0';
      }
      /** @var TCustomConnection */
      $db = $this->getDb();

      if(/** @var TDataSet */ $res = $db->query(sqlImplode($dummyQuery))) {
        $res->fetchFields();
        $x = $db->quoteFieldName($res->fieldsByIndex[1]->table)
          .'.'.$db->quoteFieldName($res->fieldsByIndex[1]->name)." LIKE '%".$db->escape($term)."%'";
        if(isset($realQuery['WHERE'])) {
          $realQuery['WHERE'] = '('.$realQuery['WHERE'].') AND '.$x;
        } else {
          $realQuery['WHERE'] = $x;
        }
      }
      $this->query = sqlImplode($realQuery);
      return true;      
    }    
    public function __construct(TComponent $parent, $name = null, $properties = null) {
      parent::__construct($parent,$name,$properties);
      $this->actions['filter'] = ACTION_BEFORE_INIT;
      $this->additionalCssClasses[] = 'rSelectBox';
    }
    public function setDataset(TDataSet $dataset)  {
      parent::setDataset($dataset);
    }
    public function loadFromDataset() {
      parent::loadFromDataset();
      if($this->relatedRows) { 
        foreach($this->getDataset()->fields as /** @var TField */ $field) {
          if($field->primaryKey) {
            foreach($this->options as $key => /** @var TSelectOption */ $option) {
              if($option->value == $field->oldValue) {
                unset($this->options[$key]);
              }
            }
          }
        }
      }
    }    
    public function init() {
      /** @var TCustomConnection */
      $db = $this->getDb();       
                     
      if(/** @var TDataSet */ $res = $db->query($this->query)) {
        while($arr = $res->fetchArray()) {
          $opt = $this->addOption($arr[0],$arr[1]); 
          if(isset($arr[2])) {
            $opt->title = $arr[2];
          }
          if(isset($arr[3])) {
            $opt->group = $arr[3];
          }
        }
      }
      if(($this->groupQuery !== null) && (/** @var TDataSet */ $res = $db->query($this->groupQuery))) {
        while($arr = $res->fetchArray()) {
          $this->addGroup($arr[0],$arr[1]);
        }
      }      
    }
  }
  /**
  *
  */
  class TPanelOpener extends TControl {
    /** @var string Name of the panel component to be open */
    public $panelName;
    /** @var bool Hide all other sibling panels?  */
    public $exclusive = false;
    /** @var int Fancybox window width in pixels */
    public $windowWidth = null;
    /** @var int Fancybox window height in pixels */
    public $windowHeight = null;
    public function __construct(TComponent $parent, $name = null, $properties = null) {
      parent::__construct($parent,$name,$properties);
      $this->road->useLibrary('fancybox');
    }    
  }
  
  abstract class TCustomPager extends TControl {
    /** @var int Total count of items */
    public $total = null;
    /** @var int Count of items per page */
    public $perPage = 20;
    /** @var int Navigator options */
    public $navigator = PAGER_NAV_NONE; // replaced with PAGER_NAV_TOP | PAGER_NAV_ITEM_NUMBERS | PAGER_NAV_NEXT_PREV;
    /** @var string Query to select items */
    public $query;
    /** @var TDataSet Dataset or just an array of data */  
    public $data = array();
    /** @var int Cout of new items */
    public $newItemCount = 0;
    /** @var int Count of pages to display in the navigator */
    public $maxShorcuts = 5;
    /** @var bool Maintain the pages (pages always starts with $perPage+1 ) */
    public $maintainPages = true;
    /** @var int */
    public $defaultPage = 1;
    private $activePage = null;
    private $firstItem = 0;
    private $displayedCount = 0;
    /** @var int Auto scroll */
    public $scrollInterval = null;
    /** @var bool */
    protected $initialized = false;
    protected $structureLoaded = false;
    /**
    * Returns active page number 
    *
    * @return int
    */
    public function getActivePage()  {
      return $this->activePage;
    }

    /**
    * Returns true if database data are linked to this pager 
    *
    * @return bool
    */
    public function isDataLinked()  {
      return is_object($this->data);
    }
    /**
    *
    *
    * @return void
    */
    protected function adjustFilter(&$where, &$order) {
    }
    /**
    * Returns index of first item to be shown 
    *
    * @return int
    */
    public function getFirstIndex()  {
      return $this->firstItem;
    }
    public function checkPageBounds($value) {
      if($value < 1) {
        $value = 1;
      } elseif($value > $this->getPageCount()) {
        $value = $this->getPageCount();
      }
      return $value;
    }
    public function checkIndexBounds($value) {
      if($value < 0) {
        $value = 0;
      } elseif($value >= $this->total) {
        $value = $this->total -1;
      }
      return $value;
    }
    /**
    * Associtive array of pages and their first and last indexes 
    *
    * @param bool True if $maxShortcuts should be respected
    * @return array[]
    */
    public function getAllPages($respectMaxShortcuts = false)  {
      $pages = array();
      if($this->total < 1) {
        return $pages;
      }
      if($this->maintainPages) {
        for($i = 1; $i <= $this->getPageCount(); $i++) {
          $pages[$i] = array(
            $this->checkIndexBounds($this->perPage*($i-1)),
            $this->checkIndexBounds($this->perPage*$i-1)
          );
        }
      } else {        
        for($i = 1; $i <= $this->getPageCount(); $i++) {
          $pages[$i] = array(
            $this->checkIndexBounds($this->getFirstIndex() + ($i - $this->getActivePage())*$this->perPage),
            $this->checkIndexBounds($this->getFirstIndex() + ($i - $this->getActivePage())*$this->perPage + $this->perPage -1)
          );
        }
        ksort($pages);      
      }
      if($respectMaxShortcuts && ($this->maxShorcuts > 0)) {
        $maxShortcuts = $this->maxShorcuts - ($this->navigator & PAGER_NAV_FIRST_LAST ? 2 : 0);
        if($maxShortcuts < 1) {
          $maxShortcuts = 1;
        }
        $maxBefore = floor(($maxShortcuts -1)/2);      
        $maxAfter = $maxShortcuts - 1 - $maxBefore;
        $r = -($this->getPageCount() - $this->getActivePage() - $maxAfter - ($this->navigator & PAGER_NAV_FIRST_LAST ? 1 : 0));
        if($r > 0) {
          if(($this->navigator & PAGER_NAV_FIRST_LAST) && ($r > 1)) $r--;
          $maxBefore += $r;
        }
        $r = -($this->getActivePage() - 1 - $maxBefore - ($this->navigator & PAGER_NAV_FIRST_LAST ? 1 : 0));
        if($r > 0) {
          if(($this->navigator & PAGER_NAV_FIRST_LAST) && ($r > 1)) $r--;
          $maxAfter += $r;
        }
        $first = $this->getActivePage()-$maxBefore;
        $last = $this->getActivePage()+$maxAfter;
        $ellipsis = false;        
        foreach(array_keys($pages) as $page) {
          if($this->navigator & PAGER_NAV_FIRST_LAST) {
            if(($page <= 1) || ($page >= $this->getPageCount())) {
              $ellipsis = false;
              continue;
            }
          }
          if(
            ($page < $first) || ($page > $last)
          ) {
            if($ellipsis) {
              unset($pages[$page]);
            } else {
              $pages[$page] = '...';
              $ellipsis = true;
            }
          } else {
            $ellipsis = false;
          }
        }
      }
      return $pages;    
    }
    /**
    * Returns index of first item to be shown 
    *
    * @return int
    */
    public function getLastIndex()  {
      return $this->firstItem+$this->getDisplayedCount()-1;
    }
    /**
    * Set active page to
    *
    * @return void
    */
    public function page($pageNumber) {
      $pageNumber = $pageNumber + 0;
      if($pageNumber < 1) {
        $pageNumber = 1;
      }      
      $this->activePage = $pageNumber;
      $this->firstItem = ($pageNumber - 1) * $this->perPage;
      return true;      
    }
    /**
    * Returns count of items to be realy displayed
    *
    * @return int
    */
    public function getDisplayedCount( )  {
      return $this->displayedCount;    
    }
    /**
    * Returns total page count
    *
    * @return int
    */
    public function getPageCount()  {
      if($this->maintainPages) {
        return ceil($this->total / $this->perPage);
      } else {
        return $this->getActivePage()+ceil(($this->total - $this->getLastIndex()-1)/$this->perPage);
      }
    }
    /**
    * Force item to be seen
    *
    * @return void
    */    
    public function item($fromItem) {
      $fromItem = $fromItem;
      if($fromItem < 0) {
        $fromItem = 0;
      }
      if($this->maintainPages) {
        $this->activePage = ceil(($fromItem+1) / $this->perPage);
        $this->firstItem = ($this->activePage - 1) * $this->perPage;      
      } else {
        $this->firstItem = $fromItem;
        $this->activePage = ceil(($fromItem) / $this->perPage)+1;        
      }
      return true;
    }
    public function __construct(TComponent $parent, $name = null, $properties = null) {
      $this->navigator = PAGER_NAV_TOP | PAGER_NAV_PAGE_NUMBERS | PAGER_NAV_NEXT_PREV;
      parent::__construct($parent,$name,$properties);
      $this->pager = $this;
      $this->actions['item'] = ACTION_BEFORE_INIT;
      $this->actions['page'] = ACTION_BEFORE_INIT;      
    }
    protected function loadStructure() {
      if($this->query) {
        /** @var TCustomConnection */
        $db = $this->getDb();
        $realQuery = sqlExplode($this->query);
        if(!$this->structureLoaded) {
          $query = $realQuery;
          $query['WHERE'] = '1 = 0';
          /** @var TDataSet */
          $res = $db->query(sqlImplode($query));
          $this->data = $res;
          $res->fetchFields();
          $this->structureLoaded = true;
        }
      }
      return $realQuery;
    }
    public function preInit()  {
      if($this->initialized) {
        return;
      }

      $this->initialized = true;
      if(isset($_GET[$this->getId().'-page'])) {
        $this->page($_GET[$this->getId().'-page']);              
      }
      if(isset($_GET[$this->getId().'-item'])) {
        $this->item($_GET[$this->getId().'-item']);              
      }
      if(isset($_POST[$this->getId().'-page'])) {
        $this->page($_POST[$this->getId().'-page']);              
      }
      if(isset($_POST[$this->getId().'-item'])) {
        $this->item($_POST[$this->getId().'-item']);              
      }


      if(isset($_GET[$this->getId().'-insert'])) {
        $this->newItemCount = $_GET[$this->getId().'-insert']+0;              
      }
      if(isset($_POST[$this->getId().'-insert'])) {
        $this->newItemCount = $_POST[$this->getId().'-insert']+0;              
      }
      $items = null;
      
      if(isset($_GET[$this->getId().'-items'])) {
        $items = $_GET[$this->getId().'-items'];
      }
      if(isset($_POST[$this->getId().'-items'])) {
        $items = $_POST[$this->getId().'-items'];
      }
      if(!is_array($items)) {
        $items = null;
      }
      $realQuery = $this->loadStructure();
      $res = $this->data;
      $filter = array();
      $order = array();
      $this->adjustFilter($filter,$order);
        
      if($this->query) {
        /** @var TCustomConnection */
        $db = $this->getDb();

        if(count($order) > 0) {
          if(isset($realQuery['ORDER BY'])) {
            $realQuery['ORDER BY'] .= ', '.implode(', ',$order);
          } else {
            $realQuery['ORDER BY'] = implode(', ',$order);
          } 
        }        
        if($items) {


          $whereQuery = array();
          $fields = $res->getKeyFields();
          $newItems = 0;
          if(count($fields) == 1) {

            foreach($items as $item) {
              if(startsStr('new_',$item)) {
                $newItems ++;
                continue;
              }
              if($x = $res->parseRowId($item)) {
                foreach($x as $fieldName => $value) {
                  /** @var TField */
                  $field = $res->getField($fieldName);
                  $field->value->startTempMode();
                  $field->value->setAsSql($value);
                  $whereQuery[] = $field->value->getAsSqlQuoted();
                  $field->value->endTempMode();
                  break;                   
                }
              }
            }
            if(count($whereQuery) > 0) {
              $whereQuery = $db->quoteFieldName($fields[0]->name).' IN ('.implode(',',$whereQuery).')';
            } else {
              $items = null;
            }
          } else {
            foreach($items as $item) {
              if(startsStr('new_',$item)) {
                $newItems ++;
                continue;
              }            
              if($x = $res->parseRowId($item)) {
                $partQuery = array();
                foreach($x as $fieldName => $value) {

                  /** @var TField */
                  $field = $res->getField($fieldName);
                  $field->value->startTempMode();
                  $field->value->setAsSql($value);
                  $partQuery[] = $db->quoteFieldName($field->name) .' = '. $field->value->getAsSqlQuoted();
                  $field->value->endTempMode();
                }
                $whereQuery[] = '('.implode(' AND ',$partQuery).')';
              }
              
            }
            if(count($whereQuery) > 0) {
              $whereQuery = '('.implode(' OR ',$whereQuery).')';
            } else {
              $items = null;
            }            
          }
          $this->newItemCount = max($newItems,$this->newItemCount);
        }

        if($this->total === null) {
          $query = $realQuery;
          $query['SELECT'] = 'COUNT(*)';        

          $res = $db->query(sqlImplode($query));
          $this->total = $res->fetchValue(0);
        }

        if($this->perPage < 1) {
          $this->perPage = $this->total;
        }
        if($this->activePage === null) {
          if($this->defaultPage > 0) {
            $this->page($this->checkPageBounds($this->defaultPage));
          } else {
            $this->page($this->checkPageBounds($this->getPageCount()+$this->defaultPage));
          }
        }        
        
        if($items && $whereQuery) {
          if(isset($realQuery['WHERE'])) {
            $realQuery['WHERE'] = '('.$realQuery['WHERE'].') AND '.$whereQuery;          
          } else {
            $realQuery['WHERE'] = $whereQuery;
          }
        } else {
          $realQuery['LIMIT'] = $this->perPage;
          $realQuery['OFFSET'] = $this->getFirstIndex();
        }                         
        $this->data = $db->query(sqlImplode($realQuery));
        $this->displayedCount = $this->data->getRowCount();        
      } elseif(is_array($this->data)) {
        if($this->total === null) {
          $this->total = count($this->data);          
        }
        if($this->perPage < 1) {
          $this->perPage = $this->total;
        }
        if($this->activePage === null) {
          if($this->defaultPage > 0) {
            $this->page($this->checkPageBounds($this->defaultPage));
          } else {
            $this->page($this->checkPageBounds($this->getPageCount()+$this->defaultPage));
          }
        }        
        $index = 0;
        $first = $this->getFirstIndex(); 
        $cnt = 0;
        foreach(array_keys($this->data) as $key) {
          if(($index < $first) || ($cnt >= $this->perPage)) {
            unset($this->data[$key]);
          } else {
            $cnt ++;
          }
          $index++;
        }
        $this->displayedCount = $cnt;
      }
    }
  }
  /**
  *
  */
  class TControlPager extends TCustomPager {
    /** @var int */
    private $index = -1;
    /** @event(TComponent $sender, int $index, bool $inserting) Next item is about to be rendered */
    public $onItemRender = false;
    /** @event(TComponent $sender, int $index, bool $inserting) Next item is proccessed */
    public $onPrepareItem = false;
    /** @var TDataSet[] Associative array of datasets for every item */
    public $datasets = array();
    private $renamed = false;
    /** @var array[] Associative array of controls for all visible rows */
    public $items = array();

    public function findComponent($name, $deep = false, $onlyForItem = false) {
      foreach($this->items as $id => $components) {
        if(($onlyForItem !== false) && ($id != $onlyForItem)) {
          continue;
        }
        foreach($components as $item) {
          if(sameText($item->name,$name)) {
            return $item;
          } elseif($deep) {
            $res = $item->findComponent($name,$deep);
            if($res) {
              return $res;
            }
          }
        }
      }
      return false;
    }
    public function loadInput(&$input) {
      foreach($this->items as $components) {
        foreach($components as /** @var TComponent */ $component) {
          $component->loadInput($input);
        }
      }
    }
    public function isErrorneous( ) {       
      if($this->errorMessage !== false) {
        return true;
      }
      foreach($this->items as $components) {
        foreach($components as /** @var TComponent */ $component) {
          if($component->isErrorneous()) {
            return true;
          }
        }
      }
      return false;
    }    
    public function allSaved()  {
      foreach($this->items as $components) {
        foreach($components as /** @var TComponent */ $component) {
          if($component->dbOptions & DB_LOAD) {
            $component->allSaved();
          }
        }
      }
    }
    public function datasetSaved()  {
      $res = true;
      foreach($this->items as $components) {      
        foreach($components as /** @var TComponent */ $component) {
          if($component->dbOptions & DB_SAVE) {
            $res = $res && $component->dataSetSaved();
          }
        }
      }
      return $res;
    }
    public function saveToDataset() {
      $res = true;
      foreach($this->items as $components) {
        foreach($components as /** @var TComponent */ $component) {
          if($component->dbOptions & DB_SAVE) {
            $res = $component->saveToDataSet() && $res;
          }
        }
      }
      return $res;
    }
    public function loadFromDataset() {
      foreach($this->items as $components) {
        foreach($components as /** @var TComponent */ $component) {
          if($component->dbOptions & DB_LOAD) {
            $component->loadFromDataset();
          }
        }
      }
    }    
    public function init() {
      foreach($this->items as $components) {
        foreach($components as /** @var TComponent */ $component) {
          $component->init();
        }
      }
      $this->loadFromDataSet();      
    }
    /**
    *
    *
    * @return void
    */
    public function insert() {

    }
    public function preInit() {
      if($this->initialized) {
        return;
      }
      parent::preInit();
      $this->index = $this->getFirstIndex();
      for($i = 1; $i <= $this->newItemCount; $i++) {
        $id = 'new_'.$i;
        $ds = null;        
        if(is_object($this->data)) {
          $this->data->insert();
          $this->prepare($id,true);
          $ds = $this->data->cloneMe();
          $this->datasets[$id] = $ds;                    
        }
        foreach($this->components as /** @var TComponent */ $c) {
          $x = $c->cloneMe(true);
          $x->setPagerId($id);
          $this->rename($x,$id);
          if($ds) {
            $x->setDataset($ds);
          }
          $this->items[$id][] = $x;
        }        
      }
      if($this->data->isInserting()) {
        $this->data->cancel();
        $this->data->first();
      }       
      while($id = $this->fetchItem()) {
        $ds = null;
        if(is_object($this->data)) {
          $ds = $this->data->cloneMe(); 
          $this->datasets[$id] = $ds;                    
        }
        foreach($this->components as /** @var TComponent */ $c) {
          $x = $c->cloneMe(true);
          $x->setPagerId($id);          
          $this->rename($x,$id);
          if($ds) {
            $x->setDataset($ds);
/*            if($x->dbOptions & DB_LOAD) {            
              $x->loadFromDataset();
            }*/
          }
          $this->items[$id][] = $x;
        }
      }      
    }
    /**
    * Prepare when created
    *
    * @param int Index
    * @param bool Inserting    
    * @return void
    */
    public function prepare($index,$inserting)  {
      $this->fireEvent('onPrepareItem',array('index'=>$index,'insert'=>$inserting));
    }
    private function rename(/** @var TComponent */ $component, $id)  {
      $component->name .= '_'.$id;
      foreach($component->components as $c) {
        $this->rename($c, $id);
      }
    }
    private function fetchItem() {
      $this->index++;
      if(is_object($this->data)) {
        $res = $this->data->fetchFields();
        if(!$res) return false;
        $res = $this->data->getRowId();        
      } elseif(is_array($this->data)) {
         $res = key($this->data);      
         if(current($this->data) === false) {
           return false;
         }
      } else {
        if($this->index > $this->getLastIndex()) {
          return false;
        }
        $res = $this->index;
      }
      $this->prepare($res,false);
      return $res;
    }
  }


  class TCustomTableLink extends TSubmitButton {
    /** @var bool Refresh form when saved with ajax request */
    public $refreshWhenAjaxed = true;
    private $reseted = false;
    /** @var string Table name */
    protected $table = null;
    /** @var string */
    protected $query = null;
    /** @var bool Is the record beeing saved as a copy? */
    protected $savingAsCopy = false;
    /** @var bool Use DB transactions to ensure data integrity */
    protected $useTransactions = true;
    /** @event(TComponent $sender) Occurs when the components are about to load their values from the dataset */
    protected $onBeforeLoad = false;
    /** @event(TComponent $sender) Occurs after the components finished loading from the dataset */
    protected $onAfterLoad = false;
    /** @event(TComponent $sender) All changes has been successfully saved to database */
    public $onSaved = false;
    /** @event(TComponent $sender) Occurs just before the dataset gets into insert state */
    public $onBeforeInsert = false;
    /** @event(TComponent $sender) Occurs when dataset got into insert state */
    public $onAfterInsert = false;
    /** @event(TComponent $sender) Occurs just before the components are saved to dataset */
    public $onBeforeSave = false;
    /** @event(TComponent $sender) Occurs when saving has been finished and transactio is about to be commited */
    public $onAfterSave = false;    
    /** @event(TComponent $sender) Occurs when data is about to be posted to the server */
    public $onBeforePost = false;
    /** @event(TComponent $sender) Occurs just after data have been posted to the server */
    public $onAfterPost = false;
    /** @var string[] Associative array of key fields and its values */    
    protected $keys = array();
    /** @var Automatically load keys from GET or POST? */
    protected $autoloadKeys = true;    
    /** @var int Allowed modes*/
    public $modes = MODE_BOTH;
    private $inserting = false;
//    /** @var bool Database link requrires keyField specification or insert command */    
//    public $required = true;
    private $initialized = false;
    private $successful = true;
    /**
    * Was the last action successful?
    *
    * @return bool
    */
    public function wasSuccessful()  {
      return $this->successful;
    }
    /**
    *
    *
    * @return bool
    */
    public function isInitialized ( )  {
      return $this->initialized;    
    }
    /**
    *
    *
    * @return void
    */
    public function uriAppendKeys($uri)  {
      $query[$this->getId().'-keys'] = $this->keys;
      return changeUrlQuery($uri,$query);
    }
    public function __construct(TComponent $parent, $name = null, $properties = null) {
      parent::__construct($parent,$name,$properties);
      $this->additionalCssClasses[] = 'rSubmitButton';
      $this->actions['insert'] = ACTION_BEFORE_INIT;
    }
    /**
    * Switch to insert mode  
    *
    * @return void
    */
    protected function insert( )  {
      if(!($this->modes & MODE_INSERT)) {
        throw new EException(E_INSERT_MODE_NOT_ALLOWED);
      }
      $this->inserting = true;
      return true;
    }
    
    /**
    * Is the insert mode active?
    *
    * @return bool
    */
    protected function isInserting() {
      return $this->inserting;
    }

    protected function postData(&$userAbort) {
      $res = true;
      if($res) {
        $res = ($this->fireEvent('onBeforePost') !== false); 
        $userAbort = !$res;
      }
      if($res) {
        $dataset = $this->getDataset();
        $res = $dataset->post();
        $previousKeys = $this->keys;
        $this->keys = array();
        foreach($dataset->fields as /** @var TField */ $field) {
          if($field->primaryKey) {
            $this->keys[$field->name] = $field->value->getAsSql();
          }
        }
      }
      if($res) {
        $res = ($this->fireEvent('onAfterPost') !== false); 
        $userAbort = !$res;
      }
      return $res;
    }
    protected function internalSave($asCopy, $formOrPager) {
      $this->savingAsCopy = $asCopy;
      $this->successful = false;
      if($asCopy || $this->isInserting()) {
        if(!($this->modes & MODE_INSERT)) {
          throw new EException(E_INSERT_MODE_NOT_ALLOWED);
        }
      } else {
        if(!($this->modes & MODE_EDIT)) {
          throw new EException(E_EDIT_MODE_NOT_ALLOWED);
        }
      }
      /** @var TCustomConnection */
      $db = $this->getDb();
      $userAbort = false;      
      if($this->useTransactions) {
        $res = $db->begin();
      } else {
        $res = true;
      }

      if($res && $asCopy) {
        $res = ($this->fireEvent('onBeforeInsert') !== false);
        $userAbort = !$res;         
        if($res) {
          /** @var TDataSet */
          $dataset = $this->getDataset();

          $dataset->insert();
          $res = ($this->fireEvent('onAfterInsert') !== false);
          $userAbort = !$res;         
        }
      }
      if($res) {
        $res = ($this->fireEvent('onBeforeSave') !== false);
        $userAbort = !$res;         
      }      
      
      if($res) {
        $res = $formOrPager->saveToDataset();
        $userAbort = !$res;
      }
      if($res) {
        $res = $this->postData($userAbort);
      }

      if($res) {
        $res = $formOrPager->datasetSaved();        
      }
      if($res) {
        $res = ($this->fireEvent('onAfterSave') !== false);        
        $userAbort = !$res;
      }
      if(!$res && $asCopy) {
        $dataset->cancel();
      }            
      if($res) {
        if($this->useTransactions) {
          $res = $db->commit();
        }
      }
      if($res) {
        $res = $formOrPager->allSaved();
        $this->inserting = false;
        $this->successful = true;
        
        if($this->fireEvent('onSaved') === null) {
          $this->road->queueMessage(LNG_CHANGES_SAVED,MSG_SUCCESS);
          if($this->refreshWhenAjaxed || !$this->road->isAjaxed()) {
            $this->road->redirect(changeUrlQuery($this->uriAppendKeys(absoluteUri(':')),array('rAction' => false),URL_QUERY_REMOVE));
          }
        }
      } else {
        if(isset($previousKeys)) {
          $this->keys = $previousKeys;
        }
        if(!$userAbort) {
          $this->setError(E_CHANGES_COULD_NOT_BE_SAVED);
        }
        if($this->panel !== null) {
          $this->panel->visible = true;        
        }
        $db->rollback();
      }
      return $res;      
    }
    /**
    * Save changes made by user to the database
    *
    * @return bool
    */
    public function save() {
      return $this->internalSave(false,$this->form);
    }

    public function click() {
      if($this->fireEvent('onClick') === true) {
        return true;
      }
      $this->save();
      return true;
    }
    public function init() {
      if($this->initialized) {
        return;
      }
      /** @var TCustomConnection */
      $db = $this->getDb();

      if($this->table !== null) {
        $this->query = 'SELECT * FROM '.$db->quoteFieldName($this->table);      
      }      
      $this->successful = false;      

      $originalQuery = sqlExplode($this->query); 
      if($originalQuery === false) {
        throw new EException(E_SQL_BAD_SYNTAX_OR_TOO_COMPLEX,array($this->query));
      }
      $dummyQuery = $originalQuery;
      $dummyQuery['WHERE'] = ' 1 = 0';
      $dummyQuery = sqlImplode($dummyQuery);

      /** @var TDataSet */      
      $dataset = $db->query($dummyQuery);
      
      $submited = array();
      
      if($this->autoloadKeys) {
        $inputName = $this->getId().'-keys';
        if(isset($_GET[$inputName])) {
          $submited = $_GET[$inputName]; 
        }
        if(isset($_POST[$inputName])) {
          if(is_array($_POST[$inputName]) && is_array($submited)) {
            $submited = array_merge($submited,$_POST[$inputName]);
          } else {
            $submited = $_POST[$inputName];
          } 
        }
        if(!is_array($submited)) {        
          $submited = $dataset->parseRowId($submited);
          if($submited == false) {
            throw new EException(E_RECORD_KEY_VALUE_MISSING);
          }
        }
      }


      if(!$this->isInserting()) {
        if((count($this->keys) == 0) && (count($submited) == 0)) {
          if($this->modes & MODE_INSERT) {
            $this->insert();
          } else {
            return false;
          }
        }
      }

            
      if($this->isInserting()) {
        if($dataset) {        
          $dataset->fetchFields();
          $this->dataset = $dataset; 
          $this->form->setDataset($dataset);
          
          if($this->fireEvent('onBeforeInsert') === false) {
            return;
          }
          $dataset->insert();
          if($this->fireEvent('onAfterInsert') === false) {
            return;
          }
          if($this->fireEvent('onBeforeLoad') === false) {
            return;
          }
          $this->form->loadFromDataset();
          $this->successful = true;          
          if($this->fireEvent('onAfterLoad') === false) {
            return;
          }
        }
      } else {      
        if($dataset === false) {
          return false;
        }
        $dataset->fetchFields();      
  
        $where = array();

        foreach($dataset->fields as /** @var TField */ $field) {
          if($field->primaryKey) { 
            $field->value->strict = true;
            if(isset($this->keys[$field->name])) {
              $field->value->setRaw($this->keys[$field->name]);          
            } else {
              if(isset($submited[$field->name])) {
                $field->value->setAsSql($submited[$field->name]);
                $this->keys[$field->name] = $field->value->getRaw();
              } else {
                throw new EException(E_RECORD_KEY_VALUE_MISSING);
              }
            }
            $where[] = $db->quoteFieldName($field->table).'.'.$db->quoteFieldName($field->name).' = '.$field->value->getAsSqlQuoted();
          }
        }
        if(count($where) == 0) {
          throw new EException(E_RECORD_KEY_VALUE_MISSING);
        }
        
        $realQuery = $originalQuery;
  
        if(isset($realQuery['WHERE'])) {
          $realQuery['WHERE'] = ' ('.$realQuery['WHERE'].') AND '.implode(' AND ',$where);
        } else {
          $realQuery['WHERE'] = ' '.implode(' AND ',$where);
        }
        $realQuery = sqlImplode($realQuery);

        if($res = $db->query($realQuery)) {
          if(/** @var TDataSet */ $res->getRowCount() < 1) {          
            throw new EException(E_RECORD_NOT_FOUND);
          }
          $res->fetchFields();
          $this->form->setDataset($res);
          $this->initialized = true;
          $this->successful = true;          
          if($this->fireEvent('onBeforeLoad') === false) {
            return;
          }
          $this->form->loadFromDataset();
          if($this->fireEvent('onAfterLoad') === false) {
            return;
          }
        }
      }
      parent::init();
    }
  }
  class TTableLink extends TCustomTableLink {
    public $table = null;
    public $query = null;
    public $savingAsCopy = false;
    public $useTransactions = true;
    public $onBeforeLoad = false;
    public $onAfterLoad = false;    
    public $keys = array();
    public $autoloadKeys = true;    
    /**
    * Save form as a new row
    *
    * @return bool
    */
    public function saveAsCopy ( )  {
      return $this->internalSave(true,$this->form);
    }
    public function isInserting() {
      return parent::isInserting();
    }
    public function insert()  {
      return parent::insert();    
    }    
  }
  class TMultiTableLink extends TCustomTableLink {
    /** @var string Name of the pager to save */
    public $pagerControl;
    
    /**
    * Get pager specified by pagerControl property
    *
    * @return TControlPager
    */
    public function getPager()  {
      if($res = $this->page->findComponent($this->pagerControl,true)) {
        return $res;
      } else {
        throw new EException(E_COMPONENT_NOT_FOUND,array($this->pagerControl));
      }
    }
    public function init() {
    }

    protected function postData(&$userAbort) {       
      $pager = $this->getPager();
      $res = true;      
      foreach($pager->datasets as /** @var TDataSet */ $dataset) {
        $this->dataset = $dataset;
        if($dataset->isInserting()) {
          if($res) {
            $res = ($this->fireEvent('onBeforeInsert') !== false); 
            $userAbort = !$res;
          }
          if($res) {
            $res = ($this->fireEvent('onAfterInsert') !== false); 
            $userAbort = !$res;
          }
        }
        if($res) {
          $res = ($this->fireEvent('onBeforePost') !== false); 
          $userAbort = !$res;
        }
        if($res) {
          $res = $dataset->post();
        }
        if($res) {
          $res = ($this->fireEvent('onAfterPost') !== false); 
          $userAbort = !$res;
        }
        if(!$res) {
          return false;
        }                
      }
      return $res;
    }

    protected function internalSave($asCopy,$formOrPager) {
      return parent::internalSave(false,$this->getPager());      
    }
    public function uriAppendKeys($uri)  {
      $pager = $this->getPager();    
      $query[$pager->getId().'-page'] = $pager->getActivePage();
      return changeUrlQuery($uri,$query);
    }
  }

  class TTagWrapper extends TControl {
    /** @var string Tag name */
    public $tag;
    /** @var string[] Custom attributes */
    public $attributes = array();
    /** @var string Inner HTML code */
    public $code;
  }

  class TLinkTag extends TSubmitButton {
    public $newRow = false;
    /** @var string */
    public $href = '#';
    /** @var string Inner HTML code */
    public $code;
    
    public function getCaption() {
      if(($this->caption === null) && ($this->href !== '#')) {
        return $this->href;
      } else {
        return parent::getCaption();
      }
    }
  }
  class TLinkButton extends TLinkTag {
    public function __construct(TComponent $parent, $name = null, $properties = null) {
      parent::__construct($parent,$name,$properties);
      $this->additionalCssClasses[] = 'rButton';
    }
  }
  class TExpandButton extends TLinkButton {
    /** @var string Name of the control to show / hide */
    public $control;
    /** @var bool slide UP instead of DOWN */
    public $slideUp = false;
    /** @var bool Slide or just show */
    public $slideEffect = false;

    public function loadInput(&$input) {
      if(isset($input[$this->getId().'-expanded'])) {
        $control = $this->page->findComponent($this->control,true);
        if(!$control) {
          throw new EException(E_COMPONENT_NOT_FOUND,array($this->control));
        }
        $control->visible = ($input[$this->getId().'-expanded'] == '1'); 
      }
    }    
  }
  /**
  *
  */
  class TButton extends TSubmitButton {
    public $allowRows = false;    
  }
  /**
  *
  */
  class TInsertButton extends TButton {
    /** @var string Name of the pager to save */
    public $pagerControl;
    /** @var int How many new rows to insert */
    public $howMany = 1;
    /**
    * Get pager specified by pagerControl property
    *
    * @return TControlPager
    */
    public function getPager()  {
      if($res = $this->page->findComponent($this->pagerControl,true)) {
        return $res;
      } else {
        throw new EException(E_COMPONENT_NOT_FOUND,array($this->pagerControl));
      }
    }  
    public function __construct(TComponent $parent, $name = null, $properties = null) {
      parent::__construct($parent,$name,$properties);
      $this->additionalCssClasses[] = 'rButton';
    }  
  } 
  class TDeleteButton extends TButton {
    public function click() {
      if($this->fireEvent('onClick') === true) {
        return true;
      }
      if(is_object($this->pager->data)) {
        $this->getDataset()->delete();
      }
      unset($this->pager->items[$this->pagerId]);

      $this->road->queueMessage(LNG_RECORD_DELETED,MSG_SUCCESS);
      if(!$this->road->isAjaxed()) {
        $this->road->redirect(changeUrlQuery(absoluteUri(':'),
          array(
            'rAction' => false,
            $this->pager->getId().'-page' => $this->pager->getActivePage()
          )
        ));
      }
      return true;
    }
    public function __construct(TComponent $parent, $name = null, $properties = null) {
      parent::__construct($parent,$name,$properties);
      $this->additionalCssClasses[] = 'rButton';
    }    
  }
  /**
  *
  */
  class THtmlData extends TComponent {
    public $newRow = false;
    /** @var bool Handle data as plain text */
    public $plainText = false;
    /** @var string HTML Code or text */
    public $data = null;
  }
  class TTextData extends THtmlData {
    public $plainText = true;
  }

  class TCheckBox extends TInputControl {
    /** @var mixed Value to be stored when checkbox is checked */
    public $valueChecked = 1;
    /** @var mixed Value to be stored when checkbox is unchecked */
    public $valueUnChecked = 0;
    /** @var bool Place the checkbox before label */
    public $placeBeforeLabel = false;
    /** @var bool Is the checkbox checked? */
    #public $checked = false;

    /**
    * Is the checkbox checked?
    *
    * @return bool
    */
    public function isChecked()  {
      return $this->_getChecked();
    }
    /**
    * Set check state
    *
    * @param bool
    */
    public function setChecked($value)  {
      $this->_setChecked($value);
    }
    protected function _getChecked()  {
      return ($this->value == $this->valueChecked);
    }
    protected function _setChecked($value)  {
      $this->value = ($value ? $this->valueChecked : $this->valueUnChecked);
    }
    public function loadInput(&$input) {
      $this->setChecked(isset($input[$this->getId()]));
    }    

  }
  
  class TListerColumn extends TFieldLinkedControl {
    public $newRow = false;
    public $allowRows = false;  
    /** @var int Column options */
    public $options = COL_DEFAULT;
    /** @var int Column options */
    public $sortOrder = SORT_NONE;

    /** @var string ORDER BY query when sorted A-Z */
    public $orderBy = null;
    /** @var string ORDER BY query when sorted Z-A */
    public $orderByDesc = null;
    /** @event(TComponent $sender, $order) Header button has been clicked */
    public $onSort = false;
    /** @event(TComponent $sender, TDataSet $data) Dynamically get cell title */
    public $onGetTitle = false;
    /** @event(TComponent $sender, TDataSet $data) Dynamically get link href */
    public $onGetHref = false;
    /** @event(TComponent $sender, TDataSet $data) Dynamically get cell data */
    public $onGetData = false;
    /** @event(TComponent $sender, $data) User formating */
    public $onFormatData = false;    
    /** @var string Internal filter string */
    public $filter = null;
    /** @var TCustomValue Internal filter string */
    public $filterFrom;
    /** @var TCustomValue Internal filter string */    
    public $filterTo;
    /** @var TLister */
    public $parent;
    /**
    * Returns cell link href based on onGetHref event 
    *
    * @return string
    */
    public function getHref() {
      return $this->fireEvent('onGetHref',array('data'=> $this->parent->isDataLinked() ? $this->parent->data : $this->parent->row));
    }
    /**
    * Returns cell title based on onGetTitle event 
    *
    * @return string
    */
    public function getTitle() {
      if($res = $this->fireEvent('onGetTitle',array('data'=>$this->parent->isDataLinked() ? $this->parent->data : $this->parent->row))) {
        return $res;
      } else {
        return $this->title;
      }
    }
    /**
    * Parse filter stored in hidden input
    *
    * @return void
    */
    public function parseFilter($value)  {

      if($this->filterFrom) {
        $value = explode(';',$value);

        if(count($value) == 2) {

          $this->filterFrom->setAsSql($value[0]);
          $this->filterTo->setAsSql($value[1]);
        }        
      } else {
        return $this->filter;      
      }    
    }
    /**
    * Encode filter to be stored in hidden input
    *
    * @return string
    */
    public function encodeFilter()  {
      if($this->filterFrom) {
        return $this->filterFrom->getAsSql().';'.$this->filterTo->getAsSql();
      } else {
        return $this->filter;      
      }
    }
//    public $menuVisible = false;    
    /**
    * Show column menu 
    *
    * @return void
    */
    public function showMenu() {
      if($this->options & COL_MENU) {
        $this->parent->menuColumn = $this;
      }
      return true;    
    }
    /**
    * Returns true, if the lister is filtered by this column
    *
    * @return bool
    */
    public function isFiltered() {
      if($this->filterFrom) {
        return !($this->filterFrom->isNull() && $this->filterTo->isNull());
      } else {
        return ($this->filter && true);      
      }
    }
    public function loadInput(&$input) {
      $result = true;
      if($this->filterFrom) {    
        if(isset($input[$this->getId().'-from'])) {
          if ($input[$this->getId().'-from'] === '') {
            $this->filterFrom->clear();
          } elseif(!$this->filterFrom->setAsInput($input[$this->getId().'-from'])) {
            $this->parent->setError($this->filterFrom->getLastError(),$this->filterFrom->getLastErrorNo());
            $result = false;
          }
        }
        if(isset($input[$this->getId().'-to'])) {
          if ($input[$this->getId().'-to'] === '') {
            $this->filterTo->clear();
          } elseif(!$this->filterTo->setAsInput($input[$this->getId().'-to'])) {
            $this->parent->setError($this->filterTo->getLastError(),$this->filterTo->getLastErrorNo());
            $result = false;            
          }
        }
      } else {
        if(isset($input[$this->getId().'-filter'])) {
          $this->filter = $input[$this->getId().'-filter']; 
        }
      }
      if($result) {   
        if($result && ($this->parent->menuColumn == $this)) {
          $this->parent->menuColumn = null;          
        }
      } else {
        $this->showMenu();      
      }
      return $result;
    }
  
    public function setDataset(TDataSet $dataset)  {
      parent::setDataset($dataset);
      $format = $this->getFormat();
      if($format !== false) {
        if($format->isInstanceOf('TDate')) { 
          $this->filterFrom = new TDate();
          $this->filterTo = new TDate();          
        } elseif($format->isInstanceOf('TTime')) {
          $this->filterFrom = new TTime();
          $this->filterTo = new TTime();        
        } elseif($format->isInstanceOf('TDateTime')) {
          $this->filterFrom = new TDateTime();
          $this->filterTo = new TDateTime();                
        } elseif($format->isInstanceOf('TNumber')) {
          $this->filterFrom = new TNumber();
          $this->filterTo = new TNumber();
        }
        if($this->filterFrom) {
          $this->filterFrom->verbose = false;
          $this->filterTo->verbose = false;
        }
      }
    }
    public function getData() {
      if($this->onGetData) {      
        if($this->parent->isDataLinked()) {
          return $this->fireEvent('onGetData',array('data'=>$this->parent->data));
        } else {
          return $this->fireEvent('onGetData',array('data'=>$this->parent->row));        
        }
      } elseif($this->parent->isDataLinked()) {
        if($f = $this->getField()) {
          return $f->value->getRaw();
        }
      } elseif($this->field !== null) {
        
        if(is_object($this->parent->row)) {
          if(isset($this->parent->row->$field)) {
            return $this->parent->row->$field;          
          }
        } elseif(is_array($this->parent->row)) {       
          if(isset($this->parent->row[$this->field])) {
            return $this->parent->row[$this->field];          
          }
        }
      }
    }
    public function getText() {
      if($this->onFormatData) {      
        return $this->fireEvent('onFormatData',array('data'=>$this->getData()));
      } elseif($this->format) {
        if($this->format->setRaw($this->getData())) {
          return $this->format->getAsDisplay();
        } else {
          return '';
        }
      } elseif(/** @var TField */ $f = $this->getField()) {
        if($this->onGetData) {
          $f->value->startTempMode();
          if($f->value->setRaw($this->getData())) {
            return $f->value->getAsDisplay();
          } else {
            return '';
          }
          $f->value->endTempMode();
        } else {
          return $f->value->getAsDisplay();
        }
      } else {
        return $this->getData();
      }
    }
    protected function doSort($order)  {
      return $this->fireEvent('onSort',array('order'=>$order));    
    }
    /**
    *
    *
    * @return void
    */
    public function handleFilterAndOrder(&$where, &$order) {
      if($this->sortOrder & (SORT_ASCENDING | SORT_DESCENDING)) {      
        if(!$this->doSort($this->sortOrder) && $this->parent->isDataLinked()) {

          if(($this->sortOrder === SORT_ASCENDING) && ($this->orderBy !== null)) {
            $order[] = $this->orderBy;            
          } elseif(($this->sortOrder === SORT_DESCENDING) && (($this->orderByDesc !== null) || ($this->orderBy !== null))) {
            if($this->orderByDesc !== null) {
              $order[] = $this->orderByDesc;
            }
            $order[] = $this->orderBy.' DESC';
          } elseif($f = $this->getField()) {            
            $q = ($f->table ? $f->table.'.' : '').$f->name;
            if($this->sortOrder === SORT_DESCENDING) {
              $q .= ' DESC';
            }
            $order[] = $q;
          }
        }
      }
    }
    public function __construct(TComponent $parent, $name = null, $properties = null) {
      parent::__construct($parent,$name,$properties);
      $this->actions['showMenu'] = ACTION_BEFORE_INPUT;
    }    
  }
  
  /**
  *
  */
  class TLister extends TCustomPager {
    public $row;
    public $options = LISTER_DEFAULT; 
    /** @var TFilterColumn */
    public $menuColumn = null;
    /** @var string[] Associative array of columns and their captions */
    public $captions = null;
    private $prefixedColumnNames = true;
    /**
    * Create columns according to actual dataset 
    *
    * @param bool If true, all column names will be prefixed with lister name and "_"
    * @return void
    */
    public function createColumns($prefixedColumnNames) {
      if(!$this->structureLoaded) {
        $this->prefixedColumnNames = $prefixedColumnNames;
        $this->loadStructure();
      }      
    }
    protected function loadStructure() {
      if(!$this->structureLoaded) {
        $res = parent::loadStructure();
        if($this->isDataLinked()) {

          if(count($this->components) == 0) {
            if($this->captions !== null) {
              $fields = array_keys($this->captions);
            } else {
              $fields = array_keys($this->data->fields);
            } 
            foreach($fields as $field) {
              if(!isset($this->data->fields[$field])) {
                continue;
              }
              /** @var TField */
              $f = $this->data->fields[$field];
              $c = new TListerColumn($this,($this->prefixedColumnNames ? $this->name.'_' : '').$f->name);
              if($this->captions !== null) {
                $c->caption = $this->captions[$f->name];
              }
/*              if($f->primaryKey) {
                $c->sortOrder = SORT_ASCENDING;
              }*/
              $c->field = $f->name;
            }
          }
          $this->setDataset($this->data);        
        } else {      
          if((count($this->components) == 0) && count($this->data) > 0) {
            $keys = array_keys($this->data);
            $row = &$this->data[$keys[0]];
  
            if($this->captions !== null) {
              $fields = array_keys($this->captions);
            } else {
              $fields = array_keys($this->row);
            }
            foreach($fields as $field) {
              if($this->captions !== null) {
                if(!isset($this->captions[$field])) {
                  continue;
                }
              }
              $c = new TListerColumn($this,$this->name.'_'.$field);
              if($this->captions !== null) {
                $c->caption = $this->captions[$field];
              }            
              $c->field = $field;
            }
          }      
        }
        return $res;
      } else {
        return parent::loadStructure();
      }

    }
    /**
    *
    *
    * @return void
    */
    protected function adjustFilter(&$where, &$order)  {      
      if(isset($_POST[$this->getId().'-order'])) {
        $x = $_POST[$this->getId().'-order'];       
      } elseif(isset($_GET[$this->getId().'-order'])) {
        $x = $_GET[$this->getId().'-order'];      
      } else {
        $x = false;
      }

      if($x) {
        if(substr($x,0,1) == '-') {
          $x_order = SORT_DESCENDING;
          $x = substr($x,1);
        } else {
          $x_order = SORT_ASCENDING;        
        }

        foreach($this->components as $c) {
          if(sameText($c->name,$x)) {
            $c->sortOrder = $x_order;            
          } else {
            $c->sortOrder = SORT_NONE;          
          }
        }
         
      }            
      foreach($this->components as $col) {
        $col->handleFilterAndOrder($where,$order);
      }
    }
        
    public function addComponent($component)  {
      if(!$component->isInstanceOf('TListerColumn')) {
        throw new EException(E_CANNOT_ADD_AS_CHILD,array($component->getClass(),'TLister')); 
      }
      parent::addComponent($component);
    }
    /**
    *
    *
    * @return bool
    */
    public function isFiltered()  {
      foreach($this->components as $col) {
        if($col->isFiltered()) {
          return true;
        }
      }
      return false;       
    }
    /**
    *
    *
    * @return void
    */
    public function setFilter($value)  {
      if($value) {
        $value = explode(';',$value);
        foreach($value as $val) {
          $val = explode(':',$val);
          if(count($val) == 2) { 
            /** @var TColumn */
            $col = $this->findComponent($val[0],false);
            if($col) {
              $col->parseFilter(urldecode($val[1]));
            }
          }
        }
      }
    }
    /**
    *
    *
    * @return string
    */
    public function getFilter()  {
      $result = array();
      foreach($this->components as /** @var TListerColumn */ $col) {
        if($col->isFiltered()) {
          $result[] = $col->getId().':'.urlencode($col->encodeFilter());
        }
      }
      return implode(';',$result);
    }    
    public function preInit()  {
      parent::preInit();
      if($this->isDataLinked()) {      
        $this->setDataset($this->data);
      }
      
    }    
  }

  class TStaticText extends TFieldLinkedControl {
    /** @var int Kind of control to display*/
    public $kind = TEXT_INLINE;
    /**
    * Returns text to be displayed
    *
    * @return string
    */
    public function getText( )  {
      if($this->caption !== null) {
        $x = $this->caption;
      } elseif($f = $this->getField()) {
        if($this->format) {
          $x = $f->value->getRaw();
        } else {
          return $f->value->getAsDisplay();
        }
      } else {
        $x = $this->getCaption();
      }
            
      if($this->format) {
        if($this->format->setRaw($x)) {
          return $this->format->getAsDisplay();
        }
      }
      return $x;
    }
  }
?>