<?php


  /**
  *
  */
  class TXhtmlPage extends TXhtmlRenderer {
    /** @var string[] Meta headers */        
    public $meta = array();
    protected $headerRendered = false;
    protected $footerRendered = false;
    public function renderContent(TComponent $component, $flags = null) {
      if(!$this->headerRendered) {
        $this->renderHeader($component,$flags);
        $this->headerRendered = true;
      }      
      $this->renderChildren($component,$flags);
      if(!$this->footerRendered) {
        $this->renderFooter($component,$flags);
        $this->footerRendered = true;        
      }    
    }
    public function beforeRender(TComponent $component, $flags = null) {
      $this->metaAddStyle('/'.relativePath(dirname(__FILE__)).'/style.css');
      $this->metaAddScript('/road/core/road.js');
    }
    /**
    * @param TPage Page
    */
    public function renderHeader(TComponent $component, $flags = null)  {
      if($this->headerRendered) {
        return;
      }
      $this->headerRendered = true;
      foreach($this->road->libraries as $lib) {
        foreach($lib as $file) {
          $ext = fileExtension($file);

          if(sameText($ext,'css')) { 
            $this->metaAddStyle($file);
          } elseif(sameText($ext,'js') && $this->road->browser->supportsJavaScript()) {
            $this->metaAddScript($file);
          }
        }
      }

/*      if($this->road->browser->supportsJavascript()) {
 
        $this->metaAddScript('/road/lib/jquery.js');
        $this->metaAddScript('/road/lib/jquery-ui.js');
//        $this->metaAddStyle('/css/jquery/jquery.cleditor.css');
//        $this->metaAddScript('/road/lib/jquery.cleditor.js');
        $this->metaAddStyle('/css/jquery/jquery-ui.css');
        $this->metaAddStyle('/css/fancybox/jquery.fancybox-1.3.1.css');
        $this->metaAddScript('/road/lib/jquery.fancybox-1.3.1.js');
        $this->metaAddScript('/road/core/road.js');        
      }*/
      $this->beforeRender($component, $flags);
      $this->output->rCode('<?xml version="1.0" encoding="'.h($this->output->encoding).'" ?>
');

      switch($this->output->doctype) {
        case DOCTYPE_STRICT :
          $this->output->rCode('<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
');
          break;
        case DOCTYPE_TRANSITIONAL :
          $this->output->rCode('<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
');        
          break;
        case DOCTYPE_FRAMSET :
          $this->output->rCode('<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Frameset//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-frameset.dtd">
');        
          break;          
        default:
          throw new EException('Invalid DOCTYPE specified');  
      }
      $this->output->rTagOpen('html',array(
        'xmlns' => 'http://www.w3.org/1999/xhtml',
        'xml:lang' => CFG_LANGUAGE,
        'lang' => CFG_LANGUAGE        
      ));
      $this->output->rTagOpen('head');

      if(CFG_REAL_BROWSER_DETECT && !$this->road->browser->browserDetected) {
        
        $this->output->rTagOpen('script', array('type'=>'text/javascript','src'=>absoluteUri('/road/lib/jquery/jquery.js')),true,false);
        $this->output->rTagClose('script',false,false);                 
        $this->output->rJavaScript('    var cookieEnabled = false;    
    if(navigator.cookieEnabled) {
      cookieEnabled = true;
    } else if (typeof navigator.cookieEnabled == "undefined"){
      document.cookie = "testcookie";
      var cookieIndex = document.cookie.indexOf("testcookie");
      if(cookieIndex >= 0) {
        cookieEnabled = true;
        cookie.splice(cookieIndex);
      }
    }
    if('.(CFG_ALLOW_URL_SESSION ? 'true' : 'true' /*'cookieEnabled'*/).') {
      var url = window.location.href;
      var query = "rBrowserCaps=1" 
        + ($.support.ajax ? "1" : "0")
        + (cookieEnabled ? "1" : "0");
      if(url.indexOf("?")>=0) {
        url = url + "&" + query;
      } else {
        url = url + "?" + query;
      }
      window.location.href = url;
    }');
        if(!CFG_FULL_PAGE_FOR_UNDETECTED_BROWSER) {
          $this->output->rLine('<meta http-equiv="refresh" content="1;url='.htmlspecialchars(
            absoluteUri('?rBrowserCaps=000')
          ).'"/>');
        }    
      }
      $this->metaAdd('Content-Language',CFG_LANGUAGE);
      $this->metaAdd('Content-Type',$this->output->contentType.'; charset='.$this->output->encoding);
      if($flags == 'undetectedBrowser') {
        $component->title = LNG_BROWSER_DETECTION;
      } else {
        $this->output->rLine(array_pop($this->meta),true);
        $this->output->rLine(array_pop($this->meta),true);      
        foreach($this->meta as $item) {
          $this->output->rLine($item,true);
        }
      }
      $this->output->rTagText('title',sprintf(CFG_PAGE_TITLE_FORMAT,$component->title));

      $this->output->rTagClose('head');
      $this->output->rTagOpen('body');
      if($flags == 'undetectedBrowser') {
        $this->output->rTagOpen('div',array('style'=>'font-family: "Arial"; font-size: 12pt'));
        $this->output->rTagText('p',LNG_BROWSER_DETECTION_IN_PROGRESS,array('style'=>'font-weight: bold; font-size: 14pt;'));
        $this->output->rTagText('p',LNG_BROWSER_DETECTION_IN_PROGRESS_TOO_LONG);
        $url = absoluteUri('?rBrowserCaps=000');
        $this->output->rLinkText($url,$url);
        $this->output->rTagClose('div');                
      }
    }
   
    /**
    * Add http-equiv meta information to the <head> section
    *
    * @param string HTTP equivalent header name  
    * @param string Content of this header
    * @return void
    */
    public function metaAddHTTPEquiv($name, $content)  {
      $this->meta['http:'.$name] = '<meta http-equiv="'.htmlspecialchars($name).'" content="'.htmlspecialchars($content).'"/>';
    }

    /**
    * Add meta information to the <HEAD> section
    *
    * @param string Meta field name  
    * @param string Content of this header
    * @return void
    */
    public function metaAdd($name, $content)  {
      $this->meta['meta:'.$name] = '<meta name="'.htmlspecialchars($name).'" content="'.htmlspecialchars($content).'"/>';
    }

    /**
    * Add custom head item
    *                                 
    * @param string Header text 
    * @return void
    */
    public function metaAddCustom($header) {
      $this->meta[] = $header;            
    }
    /**
    * Link a script file with this page
    *                                 
    * @param string Script filename  
    * @param string Content-type of the file
    * @return void
    */
    public function metaAddScript($filename, $contentType = "text/javascript" )  {
      $filename = absoluteURI($filename);
      $this->meta['script:'.$filename] = '<script type="'.htmlspecialchars($contentType).'" src="'.htmlspecialchars($filename).'"></script>';            
    }    

    /**
    * Link a file file with this page
    *                                 
    * @param string Filename  
    * @param string File's relation to this page
    * @return void
    */
    public function metaAddLink($filename, $relation, $contentType)  {
      $filename = absoluteURI($filename);
      $this->meta['link:'.$filename] = '<link rel="'.$relation.'" type="'.htmlspecialchars($contentType).'" href="'.htmlspecialchars($filename).'" />';
    }

    /**
    * Link a style definition file with this page
    *                                 
    * @param string Script filename  
    * @param string Content-Type of the file
    * @return void
    */
    public function metaAddStyle($filename, $contentType = 'text/css')  {
      $this->metaAddLink($filename,'stylesheet',$contentType);            
    }

    /**
    * Link a FEED file with this page
    *                                 
    * @param string Script filename  
    * @param string Content-Type of the file
    * @return void
    */
    public function metaAddFeed($filename, $contentType = 'application/rss+xml')  {
      $this->metaAddLink($filename,'alternate',$contentType);            
    }
        
    /**
    * @param TPage Page
    */    
    public function renderFooter(TComponent $component)  {
      if($this->footerRendered) {
        return;
      }    
      $this->output->rTagClose('body');
      $this->output->rTagClose('html');
      $this->footerRendered = true;
    }              
  }

  class TXhtmlDummyPage extends TXhtmlPage {
    public function renderHeader(TComponent $component,$flags = null)  {
      if($this->headerRendered) return;
      parent::renderHeader($component,$flags);
      $this->output->renderMessages();
    }
  }

  /**
  *
  */
  class TXhtmlForm extends TXhtmlRenderer {
    private $errorComponents = array();
    private function findErrorComponents(TComponent $component) {
      if($component->getErrorMessage() !== false) {
        $this->errorComponents[] = $component;
      }
      if($component->isInstanceOf('TControlPager')) {
        foreach($component->items as $components) {
          foreach($components as $c) {
            $this->findErrorComponents($c);
          }
        }
      } else {
        foreach($component->components as $c) {
          $this->findErrorComponents($c);
        }
      }
    }    
    /**
    * @param TForm
    */
    public function renderHeader(TComponent $component)  {
      $attributes = $this->getAttributes($component);
      $attributes['method'] = ($component->method == FORM_METHOD_GET ? 'get' : 'post');
      $attributes['action'] = absoluteUri($component->action,CFG_ABSOLUTE_SITE_URI); 
      $this->output->rTagOpen('form',$attributes);
      $this->output->rTagOpen('div');
/*      if(($component->panel !== null) && ($component->panel->visible)) {
        $this->output->rTagShort('input',array(
          'class' => 'rHiddenKeys',
          'type' => 'hidden',
          'name' => 'rAction[]',
          'value' => $component->panel->getId().'-show()'
        ));
      }*/
      

      $this->output->rTagShort('input',array(
        'type' => 'hidden',
        'name' => 'roadFormSubmited[]',
        'value' => $component->name,
        'id' => 'road'.$component->name.'Submited'
      ));
      if($component->defaultButton) {
        $this->output->rTagShort('input',array(
          'type' => 'submit',
          'class' => 'rDummyDefaultButton',
          'tabindex' => '-1',          
          'style' => 'width: 1px; height: 1px; left: -1000px; position: absolute;',
          'name' => ':'.$component->defaultButton->getId().'-click()',
          'value' => '',
          'id' => $component->getId().'-defaultButton'
        ));
        if($this->road->browser->supportsJavaScript()) {
          $this->output->rJQuery('$("#'.$component->getId().'-defaultButton").click(function (event) {            
            event.preventDefault();
            $("#'.$component->defaultButton->getId().'").click();
          });');
        }
      }
      if(CFG_INPUT_ERROR_REPORTING & INPUT_ERROR_LIST) {
        $this->findErrorComponents($component);
        if(count($this->errorComponents) > 0) {
          $this->output->rTagOpen('ul',array('class' => 'rInputErrorList'));
          foreach($this->errorComponents as /** @var TComponent */ $c) {
            $this->output->rTagCode('li','<span>'.($c->isInstanceOf('TControl') ? htmlspecialchars($c->getCaption()) : htmlspecialchars($c->name)).':</span> '.htmlspecialchars($c->getErrorMessage()));
          }
          $this->output->rTagClose('ul');
        }
      }
      $this->output->rTagClose('div');      
    }
        
    /**
    * @param TForm
    */    
    public function renderFooter(TComponent $component)  {
      $this->output->rTagClose('form');
    } 
  }

  require_once('base.php');

  /**
  *
  */
  class TXhtmlSubmitButton extends TXhtmlRenderer {
    /**
    * @param TSubmitButton
    */
    public function renderJavaScriptEnhancements($component)  {
      if($this->road->browser->supportsJavaScript()) {
        if($component->allowAjax()) {
          $this->output->rComponentJQuery($component,'$(this).dynamicSubmit('
          .($component->confirmText ? '{confirm: "'.$this->output->escapeJSText($component->confirmText).'"}':'')
          .');');
        } elseif($component->confirmText) {
          $this->output->rComponentJQuery($component,'$(this).click(function(event) {
            if(!confirm("'.$this->output->escapeJSText($component->confirmText).'")) {
              event.preventDefault();
            }
          });');
        }
      }
    }    
    /**
    * @param TSubmitButton
    */
    public function renderContent(TComponent $component, $flags = null)  {
      $attributes = $this->getAttributes($component);
      $attributes['name'] = ':'.$component->name.'-click()';
      $attributes['type'] = 'submit';
      $attributes['value'] = $component->getCaption();      
      $this->output->rTagShort('input',$attributes);
      $this->output->rInputErrorBox($component);
      $this->renderJavaScriptEnhancements($component);
    }
 
  }

  /**
  *
  */
  class TXhtmlEditBox extends TXhtmlRenderer {    
    /**
    * @param TEditBox
    */
    public function renderContent(TComponent $component, $flags = null)  {
      if(($component->width === null) && ($component->autoSize)) {
        if($f = $component->getFormat()) {

//          /** @var TNumber */
//          $f = $component->format;
          if($f->isInstanceOf('TText')) {
            if($f->maxLength !== null) {;
              $component->width = min($f->maxLength,CFG_MAX_EDITBOX_AUTO_WIDTH).'ex';              
            }
          } elseif($f->isInstanceOf('TDateTime')) {
            $component->width = (strLength($f->getInputExample())+1).'ex';
          } elseif($f->isInstanceOf('TCurrency')) {
            $component->width = '15ex';
          } elseif($f->isInstanceOf('TNumber')) {
            if($f->maxValue !== null) {
              $w = ceil(log10($f->maxValue))+2;
            } else {
              $w = 10;
            }
            if($f->inputFormat->decimals < 0) {
              $w += 4 + strLength($f->inputFormat->decimalPoint);
            } 
            if($f->inputFormat->decimals > 0) {
              $w += $f->inputFormat->decimals + strLength($f->inputFormat->decimalPoint);
            }            
            $component->width = $w.'ex';
          } elseif($f->isInstanceOf('TBoolean')) {
            $component->width = (max(strLength(LNG_YES),strLength(LNG_NO))+1).'ex';
          }
        } elseif(/** @var TField */ $field = $component->getField()) {
          $component->width = $field->length.'ex';
        }
      }

      if($component->displayCaption) {      
        $this->renderLabel($component);
      }
      $attributes = $this->getAttributes($component,ATTRIBUTE_ALL);
      if($format = $component->getFormat()) {
        $attributes['class'] .= ' '.$format->getClass();        
      }

      if($component->notBlank) {
        $attributes['class'] .= ' rNotBlank';      
      }
      $attributes['name'] = $component->name;
      if($component->passwordInput) {
        $attributes['type'] = 'password';
      } else {
        $attributes['type'] = 'text';
      }
      if(!$component->passwordInput || $component->keepPassword) {
        $attributes['value'] = $component->value;
      }
      if(!$component->browserAutoComplete && !$this->road->browser->supportsJavaScript()) {
        // we have to use xhtml invalid autocomplete attribute
        $attributes['autocomplete'] = 'off';        
      }
      $this->output->rTagShort('input',$attributes);

      $this->output->rInputErrorBox($component);
      
      if($this->road->browser->supportsJavaScript()) {
        if(!$component->browserAutoComplete) {
          $this->output->rComponentJQuery($component,'$(this).attr("autocomplete","off");');
        }      
        if($component->autoComplete->enabled) {
          if(!$component->dynamic) {
            $component->fetchList();
            if(count($component->autoCompleteList) < 1) {
              return;
            }
          }
          $options = $component->autoComplete->toJson();
          if($component->dynamic) {
            $options->dynamic = true;
          } else {
            $options->list = $component->toJson('autoComplete'); 
          }
          $this->output->rComponentJQuery($component,'$(this).autocompletion('.json_encode($options).');');             
        }
        if($component->datePicker->enabled && (/** @var TDateTime*/ $format = $component->getFormat()) && ($format->isInstanceOf('TDateTime'))) {
          $opts = $component->datePicker->toJson();
          $opts->dateFormat =  $component->datePicker->convertFormat($format->inputFormat);
          if($val = $format->getMinValue()) {
            $format->startTempMode();
            $format->setRaw($val);
            $opts->minDate = $format->getAsInput();
            $format->endTempMode();
  
          }
          if($val = $format->getMaxValue()) {
            $format->startTempMode();
            $format->setRaw($val);
            $opts->maxDate = $format->getAsInput();
            $format->endTempMode();          
          }
          $this->output->rComponentJQuery($component,'$(this).datepicker('.json_encode($opts).');');        
        }
      }
    } 
  }


  class TXhtmlTextArea extends TXhtmlRenderer {    
    /**
    * @param TTextArea
    */
    public function renderContent(TComponent $component, $flags = null)  {
      if($component->displayCaption) {      
        $this->renderLabel($component);
      }
      $attributes = $this->getAttributes($component,ATTRIBUTE_ALL);
      if($format = $component->getFormat()) {
        if(isset($attributes['class'])) {
          $attributes['class'] .= ' '.$format->getClass();
        } else {
          $attributes['class'] .= $format->getClass();
        }        
      }

      $attributes['name'] = $component->name;
      $attributes['rows'] = $component->rows;
      $attributes['cols'] = $component->cols;      

      $this->output->rTagText('textarea',$component->value,$attributes);

      $this->output->rInputErrorBox($component);

    } 
  }
  
  class TXhtmlSelectBox extends TXhtmlRenderer {
    /**
    * @param TSelectOption
    * @param TSelectBox
    */        
    private function renderOption($option, $select, $type = SELECTBOX_STANDARD) {
      $attributes = $this->getAttributes($option,ATTRIBUTE_ALL);
      if($option->isSelected()) {
        if($type & SELECTBOX_RADIOGROUP) {
          $attributes['checked'] = true;
        } else {
          $attributes['selected'] = true;        
        }
      }

      $attributes['value'] = $option->value;
      if($type & SELECTBOX_RADIOGROUP) {
        if($select->isMultiSelect()) {
          $attributes['name'] = $select->getId().'[]';
          $attributes['type'] = 'checkbox';
        } else {
          $attributes['name'] = $select->getId();
          $attributes['type'] = 'radio';
        }
        $labOpt = $this->getAttributes($option,ATTRIBUTE_STANDARD);
//        if($type == SELECTBOX_RADIOGROUP_VERTICAL) {
          $divOpt = $labOpt;
          $divOpt['id'] .= '-row';
          $this->output->rTagOpen('div',$divOpt);
//        }*/
        $this->output->rTagShort('input',$attributes);
        $attributes = $this->getAttributes($option,ATTRIBUTE_STANDARD);
        $labOpt['for'] = $labOpt['id'];
        $labOpt['id'] .= '-label';
        $this->output->rTagText('label',$option->getCaption(),$labOpt);
        $this->output->rTagClose('div');        
      } else {
        $this->output->rTagText('option',$option->getCaption(),$attributes);        
      }      

    }
    /** @param TSelectBox */
    private function renderOptions($component, $type = SELECTBOX_STANDARD) {
      if(count($component->groups) > 0) {
        $groups = array();
        foreach($component->options as $option) {        
          if(($component->displayStyle & SELECTBOX_LINKER) && $component->allowAjax()) {
            if(!$option->isSelected()) {
              continue;
            }
          }        
          $grp = $option->group;
          if($grp !== null) {
            $found = false;
            foreach($component->groups as $group) {
              if($group->value == $grp) {
                $found = true;
                break;              
              }
            }
//            if(!isset($component->groups[$group])) {          
            if(!$found) {
              triggerError(E_SELECT_BOX_GROUP_NOT_DEFINED,array($grp,$component->name),E_USER_WARNING);
              $grp = null;
            }
          }           
          $groups[$grp][] = $option;
        }      
        if(isset($groups[null])) {
          foreach($groups[null] as $option) {
            $this->renderOption($option,$component,$type);
          }
        }
        
        foreach($component->groups as $group) {
          $gi = $group->value;
          if(!($component->showEmptyGroups || isset($groups[$gi]))) {
            continue;
          }
          $attributes = $this->getAttributes($group);
          if($type & SELECTBOX_RADIOGROUP) {
            $this->output->rTagOpen('fieldset',$attributes);
            $this->output->rTagText('legend',$group->getCaption());
          } else {
            $attributes['label'] = $group->getCaption();  
            $this->output->rTagOpen('optgroup',$attributes);
          }

 
          if(isset($groups[$gi])) {
            foreach($groups[$gi] as $option) {
              $this->renderOption($option,$component,$type);
            }
          }
          if($type & SELECTBOX_RADIOGROUP) {
            $this->output->rTagClose('fieldset');
          } else {
            $this->output->rTagClose('optgroup');
          }
        }        
      } else {
        foreach($component->options as $option) {
//          var_dump($component->allowAjax()); 
          if(($component->displayStyle & SELECTBOX_LINKER) && $component->allowAjax()) {
            if(!$option->isSelected()) {
              continue;
            }
          }        
          $this->renderOption($option,$component,$type);
        }      
      }
    }    
    /**
    * @param TSelectBox
    */
    public function renderContent(TComponent $component, $flags = null)  {
      if($component->displayStyle & SELECTBOX_RADIOGROUP) {
        if(($component->rows > 1) && ($component->height === null)) {
          $component->height = ($component->rows * 2.7).'ex'; 
        }
        $attributes = $this->getAttributes($component,ATTRIBUTE_ALL);
        
        if($component->isMultiSelect()) {        
          $name = $component->name.'[]';
        } else {      
          $name = $component->name;
        }

        if($component->displayCaption) {
          $this->renderLabel($component);
        }
        $attributes['class'] .= ' rSelectBox-radiogroup';
        if($component->displayStyle & SELECTBOX_RADIOGROUP_VERTICAL) {
          $attributes['class'] .= ' rSelectBox-radiogroup-vertical';
        } else {
          $attributes['class'] .= ' rSelectBox-radiogroup-horizontal';
        }
        $this->output->rTagOpen('div',$attributes);
        $this->renderOptions($component, $component->displayStyle);
        $this->output->rTagClose('div');      
      } else {
        $attributes = $this->getAttributes($component,ATTRIBUTE_ALL);
        if($format = $component->getFormat()) {
          if(isset($attributes['class'])) {
            $attributes['class'] .= ' '.$format->getClass();
          } else {
            $attributes['class'] = $format->getClass();
          }        
        }
        if($component->rows > 1) {
          $attributes['size'] = $component->rows;
        }
 
        if($component->isMultiSelect()) {        
          $attributes['multiple'] = true;        
          $attributes['name'] = $component->name.'[]';
        } else {      
          $attributes['name'] = $component->name;
        }
        if($component->displayCaption) {
          $this->renderLabel($component);
        }
        $this->output->rTagOpen('select',$attributes);
        $this->renderOptions($component,SELECTBOX_STANDARD);      
  
        $this->output->rTagClose('select');
      }
      $this->output->rInputErrorBox($component);

      if($this->road->browser->supportsJavaScript()) {
        if($component->displayStyle & SELECTBOX_COMBOBOX) {

          if(
            ($component->rows > 1)
            ||
            $component->isMultiSelect()
            ||
            ($component->displayStyle & SELECTBOX_RADIOGROUP)            
          ) {
            $opt = new stdClass();
            $opt->multiselect = $component->isMultiSelect();
//            $opt->select = LNG_SEARCHBOX_SELECT;
            $opt->all = LNG_SEARCHBOX_SELECT_ALL;
            $opt->none = LNG_SEARCHBOX_SELECT_NONE;
            $opt->inverse = LNG_SEARCHBOX_SELECT_INVERSE;
            
            $opt->allTitle = LNG_SEARCHBOX_SELECT_ALL_TITLE;
            $opt->noneTitle = LNG_SEARCHBOX_SELECT_NONE_TITLE;
            $opt->inverseTitle = LNG_SEARCHBOX_SELECT_INVERSE_TITLE;

            $opt->unfilter = LNG_SEARCHBOX_UNFILTER;

//            $opt->visible = LNG_SEARCHBOX_SELECT_VISIBLE;            
            $opt->filter = LNG_SEARCHBOX_FILTER;
            $this->output->rComponentJQuery($component,'$(this).searchbox('.json_encode($opt).');');        
          } else {
            $this->output->rComponentJQuery($component,'$(this).combobox();');
          }
        } elseif($component->displayStyle & SELECTBOX_LINKER) {
            $this->output->rComponentJQuery($component,'$(this).linker({dynamic: '.($component->dynamic ? 'true' : 'false')
              .', emptyMessage:'.json_encode(LNG_LINKER_NOTHING_SELECTED).'});'
            );
        }
      }
    }
 
  }
  
  
  class TXhtmlGroupBox extends TXhtmlRenderer {
    /**
    * @param TGroupBox
    */
    public function renderHeader(TComponent $component)  {
      $attributes = $this->getAttributes($component);
      if($component->displayCaption && !$component->isInstanceOf('TToolBar')) {      
        $this->output->rTagOpen('fieldset',$attributes);
        if($component->expandable && !$component->expanded && $component->isErrorneous()) {
          $component->expanded = true;
        }
        $this->output->rTagText('legend',$component->getCaption());

        if($component->expandable) {
          $this->output->rTagShort('input',array(
            'type' => 'hidden',
            'id' => $component->getId().'-expanded',
            'name' => $component->getId().'-expanded',
            'value' => ($component->expanded ? '1' : '0')
          ));
          if($this->road->browser->supportsJavaScript()) {
            $this->output->rComponentJQuery($component,'$(this).expandableGroup();');
          }
        }
        
      } else {
        $this->output->rTagOpen('div',$attributes);
        if($component->displayCaption) {
          $this->output->rTagText('span',$component->getCaption(),array('class' => 'rGroupCaption'));        
        }        
      }
    }
        
    /**
    * @param TGroupBox
    */    
    public function renderFooter(TComponent $component)  {
      if($component->displayCaption && !$component->isInstanceOf('TToolBar')) {
        $this->output->rTagClose('fieldset');
      } else {
        $this->output->rTagClose('div');
      }      
    } 
  }  

  class TXhtmlToolBar extends TXhtmlRenderer {
    /**
    * @param TGroupBox
    */
    public function renderHeader(TComponent $component)  {
      $attributes = $this->getAttributes($component);
      $this->output->rTagOpen('div',$attributes);
      if($component->displayCaption) {
        $this->output->rTagText('span',$component->getCaption(),array('class' => 'rToolBarCaption'));
      }
    }
        
    /**
    * @param TGroupBox
    */    
    public function renderFooter(TComponent $component)  {
      $this->output->rTagClose('div');
    } 
  }
  class TXhtmlPanel extends TXhtmlRenderer {
    /**
    * @param TPanel
    */
    public function renderHeader(TComponent $component)  {
//      $modal = ($component->modal && !$this->road->isAjaxed() && $this->road->browser->supportsJavascript()); 
      $attributes = array(
        'class' => 'rPanel'.($component->modal ? ' rModalPanel' : ''),
        'id' => $component->getId()
      );
      if($component->modal && !$this->road->isAjaxed() && $this->road->browser->supportsJavascript()) {
        $attributes['style'] = 'display: none;';
      }
      $this->output->rTagOpen('div',$attributes);
    }
        
    /**
    * @param TPanel
    */    
    public function renderFooter(TComponent $component)  {
      $this->output->rTagClose('div');
      if($component->modal && !$this->road->isAjaxed() && $this->road->browser->supportsJavascript()) {
        $this->output->rComponentJQuery($component,'$(this).showAsFancyBox();})');
      }
    }
/*    public function renderContent(TComponent $component, $flags = null) {
      if($component->visible) {
        parent::renderContent($component, $flags);
      }
    }*/     
  }  
  
  /**
  *
  */
  class TXhtmlTableLink extends TXhtmlSubmitButton {
    /**
    * @param TTableLink
    */
    public function renderContent(TComponent $component, $flags = null)  {
      if($component->isInserting()) {
/*        $this->output->rTagShort('input',array(
          'type' => 'hidden',
          'name' => 'rAction[]',
          'value' => $component->getId().'-insert()'
        ));*/
      } elseif($component->isInitialized()) {
        foreach($component->getDataset()->fields as /** @var TField */ $field) {
          if($field->primaryKey) {
            $this->output->rTagShort('input',array(
              'type' => 'hidden',
              'name' => $component->getId().'-keys['.$field->name.']',
              'class' => 'rHiddenKeys',
              'value' => $field->value->getAsSql()
            ));
          }
        }
      }
      parent::renderContent($component,$flags);
    }
  }
  /**
  *
  */
  class TXhtmlPanelOpener extends TXhtmlRenderer {
    /**
    * @param TPanelOpener
    */
    public function renderContent(TComponent $component, $flags = null)  {
      if( /** @var TPanel */ $panel = $component->page->findComponent($component->panelName,true)) {
        $attibutes = $this->getAttributes($component);
        $this->output->rLinkStart(
          '?rAction[]='.$panel->getId().'-show('.($component->exclusive ? '1':'').')',
          $attibutes      
        );
        $this->output->rText($component->getCaption());
        $this->output->rTagClose('a');
        
        if($component->allowAjax()) {
          $options = array(
            'autoScale: false',
            'scroll: "auto"',
            'showNavArrows: false',
            'ajax: {type: "POST",data: fields}'
          );
          if($component->windowHeight || $component->windowWidth) {
            $options[] = 'autoDimensions: false';
            if($component->windowWidth) {          
              $options[] = 'width: '.$component->windowWidth;
            }
            if($component->windowHeight) {
              $options[] = 'height: '.$component->windowHeight;
            }
          }
          
          $this->output->rJQuery('
            var self = $( "#'.$component->getId().'" ); 
            var form = self.parents("form:first");
            if(form.length > 0) {
              var fields = form.serializeArray();
            } else {
              var fields = new Array();
            }
            fields.push({
              name: "rAction[]",
              value: "'.$panel->getId().'-show('.($component->exclusive ? '1':'').')"
            });
            fields.push({
              name: "rAction[]",
              value: "'.$panel->getId().'-render()"
            });
            self.fancybox({
              '.implode(',',$options).'
            });
          ');        
        }        
      } else {
        triggerError(E_COMPONENT_NOT_FOUND,array($component->panelName),E_USER_WARNING);
      }      
    }
  } 
  
  /**
  *
  */

  class TXhtmlCustomPager extends TXhtmlRenderer {
    /** @var array[] */
    protected $data = array();
    /**
    * @param TCustomPager
    */
    protected function rPageLink($pager,$code,$page,$title)  {
      $page = $pager->checkPageBounds($page);
      $data = $this->data;
      $data[$pager->getId().'-page'] = $page;  
      $this->output->rTagCode('a',$code,array(
        'class' => 'rPager-navigation'.($pager->getActivePage() == $page ? ' rPager-activePage' : ''),
        'title' => $title,
        'href' =>
          changeUrlQuery(
            absoluteUri(':',CFG_ABSOLUTE_SITE_URI),
            $data
          )
      ));
    }
    protected function getNavigatorStyle($type) {
      $class = array();
      if($type & PAGER_NAV_TOP) {
        $class[] = 'rPager-navigated-top';
      } 
      if($type & PAGER_NAV_BOTTOM) {
        $class[] = 'rPager-navigated-bottom';
      } 
      if($type & PAGER_NAV_LEFT) {
        $class[] = 'rPager-navigated-left';
      } 
      if($type & PAGER_NAV_RIGHT) {
        $class[] = 'rPager-navigated-right';
      }
      return implode(' ',$class);
    }
    /**
    * @param TCustomPager
    */
    public function renderNavigator(TComponent $component, $type, $flags = null) {
      if(!($type & $component->navigator)) return;
      switch($type) {
        case PAGER_NAV_TOP: $c = '-top'; 
          break;
        case PAGER_NAV_LEFT: $c = '-left'; 
          break;
        case PAGER_NAV_RIGHT: $c = '-right'; 
          break;
        case PAGER_NAV_BOTTOM: $c = '-bottom'; 
          break;                    
        default: $c = '';
          break;  
      }
      $this->output->rTagOpen('div',array(
        'class' => 'rPager-navigator rPager-navigator'.$c
      ));
      $pages = $component->getAllPages(true);

      if($component->navigator & PAGER_NAV_NEXT_PREV) {
        $this->rPageLink($component,($type & (PAGER_NAV_LEFT | PAGER_NAV_RIGHT) ? '&#9650;' : ($this->output->type == OUTPUT_MOBILE ? '&lt;' : '&#9668;')),$component->getActivePage()-1,LNG_PAGER_PREV);
      }
      if($component->navigator & (PAGER_NAV_PAGE_NUMBERS | PAGER_NAV_ITEM_NUMBERS)) { 
        foreach($pages as $page => $bounds) {
          if($bounds == '...') {
            $this->output->rTagCode('span',
              ($type & (PAGER_NAV_LEFT | PAGER_NAV_RIGHT) ? '&#8942;' : '&#0133;'),
              array(
                'class' => 'rPager-ellipsis'
              )
            );
            continue;
          }

          $pg = ($component->navigator & PAGER_NAV_PAGE_NUMBERS
              ? $page
              : ($bounds[0]+1).($bounds[1] > $bounds[0] ? ' &ndash; '.($bounds[1]+1) : '')
          );
          $pgt = ($component->navigator & PAGER_NAV_PAGE_NUMBERS
              ? $page
              : ($bounds[0]+1).($bounds[1] > $bounds[0] ? ' - '.($bounds[1]+1) : '')
          );          
          $this->rPageLink($component,
            $pg,
            $page,
            ($component->navigator & PAGER_NAV_PAGE_NUMBERS
              ? sprintf(LNG_PAGER_DISPLAY_PAGE,$pgt)
              : sprintf(LNG_PAGER_DISPLAY_ITEMS,$pgt)
            )
          );
        }        
      }

      if($component->navigator & PAGER_NAV_NEXT_PREV) {
        $this->rPageLink($component,($type & (PAGER_NAV_LEFT | PAGER_NAV_RIGHT) ? '&#9660;' : ($this->output->type == OUTPUT_MOBILE ? '&gt;' : '&#9658;')),$component->getActivePage()+1,LNG_PAGER_NEXT);
      }
      $this->output->rTagClose('div');      
    }
    /**
    * @param TCustomPager
    */
    public function renderContent(TComponent $component, $flags = null) {
      $code = '';
      $this->output->rTagShort('input',array(
        'type' => 'hidden',
        'name' => $component->getId().'-page',
        'value' => $component->getActivePage()
      ));      
      if($component->allowAjax()) {
        $code .= ' $("#'.$component->getId().' > div.rPager-navigator a.rPager-navigation").click(function(event) {
          event.preventDefault();
          $("#'.$component->getId().'").refresh({url: this.href, leaveOut: ["'.$component->getId().'-items", "'.$component->getId().'-page"]});
        });';
        if(($component->getPageCount() > 0) && ($component->scrollInterval !== null) && (($this->output->type !== OUTPUT_MOBILE) || CFG_MOBILE_AUTO_REFRESH)) {
          $page = $component->getActivePage()+1;
          if($page > $component->getPageCount()) {
            $page = 1;
          }
          $code .= ' $("#'.$component->getId().'")
            .delay('.$component->scrollInterval.')
            .queue(function() {
               $(this).refresh({data: [{name:"'.$component->getId().'-page", value: "'.$page.'"}]});
            });
          ';
        }
      }
      if($code) {
        $this->output->rJQuery($code);
      } 
    }
  }

  class TXhtmlControlPager extends TXhtmlCustomPager {
    /**
    * @param TControlPager
    */
    public function renderContent(TComponent $component, $flags = null) {
      $attributes = $this->getAttributes($component);
      $attributes['class'] .= ' '.$this->getNavigatorStyle($component->navigator); 
      $this->output->rTagOpen('div',$attributes);
      $this->renderNavigator($component,PAGER_NAV_TOP,$flags);
      $this->renderNavigator($component,PAGER_NAV_LEFT,$flags);
      $this->renderNavigator($component,PAGER_NAV_RIGHT,$flags);
      $this->output->rTagOpen('div',array(
        'id' => $component->getId().'-content',
        'class' => 'rPager-content'
      ));

      foreach($component->items as $id => $components) {
        $this->output->rTagOpen('div',array(
          'id' => $component->getId().'-'.$id,
          'class' => 'rPager-item'
        ));      
        $component->fireEvent('onItemRender',array($id));
        foreach($components as /** @var TComponent */ $c) {
          $c->render();
        }
        $this->output->rTagClose('div');        
      }
      $this->output->rTagClose('div');
      $this->renderNavigator($component,PAGER_NAV_BOTTOM,$flags);
      foreach(array_keys($component->items) as $id) {
        $this->output->rTagShort('input',array(
          'type' => 'hidden',
          'name' => $component->getId().'-items[]',
          'value' => $id
        ));
      }      
      parent::renderContent($component,$flags);
      if($component->navigator & (PAGER_NAV_LEFT | PAGER_NAV_RIGHT)) {
        $this->output->rTagCode('div','&nbsp;',array(
          'class' => 'rClearer',
          'style' => 'clear: '.
          (($component->navigator & PAGER_NAV_LEFT) && ($component->navigator & PAGER_NAV_RIGHT)
          ? 'both'
          : ($component->navigator & PAGER_NAV_LEFT
            ? 'left'
            : 'right'
            )
          )
          .'; font-size: 1px'
        ));
      }
      $this->output->rTagClose('div');                  
    }
  }

  class TXhtmlTagWrapper extends TXhtmlRenderer {
    /**
    * @param TTagWrapper
    */
    public function renderContent(TComponent $component,$flags = null)  {
      $attributes = $this->getAttributes($component);
      $attributes = array_merge($attributes,$component->attributes);
      $type = $this->output->getTagType($component->tag,false);
      if($type & TAG_INLINE) {
        $component->allowRows = false;
      }
      if((count($component->components) < 1) && ($component->code === null) && ($type & TAG_SHORT)) {
        $this->output->rTagShort($component->tag,$attributes);
      } else {
        $this->output->rTagOpen($component->tag,$attributes);
        if($component->code !== null) {
          $this->output->rCode($component->code);
        } elseif(count($component->components) > 0) {
          $this->renderChildren($component,$flags);
        } else {
          $this->output->rText($component->getCaption());
        }
        $this->output->rTagClose($component->tag);
      }
    }
  }

  class TXhtmlLinkTag extends TXhtmlRenderer {
    /**
    * @param TLinkTag
    */
    public function renderContent(TComponent $component,$flags = null)  {
      $attributes = $this->getAttributes($component);
      $this->output->rLinkStart($component->href,$attributes);
      if($component->code !== null) {
        $this->output->rCode($component->code);
      } else {
        $this->output->rText($component->getCaption());
      }
      $this->output->rTagClose('a');
    }
  }
  class TXhtmlExpandButton extends TXhtmlLinkTag {
    /**
    * @param TExpandButton
    */
    public function renderContent(TComponent $component,$flags = null)  {
      $code = $component->code;
      $control = $component->control;
      if($component->pager !== null) {
        $control .= '_'.$component->pagerId; 
      }
      if(!($control = $component->page->findComponent($control,true))) {
        throw new EException(E_COMPONENT_NOT_FOUND,array($control));
      } 
      if((!$control->visible && $component->slideUp) || ($control->visible && !$component->slideUp)) {
        $char = '&#9650;';
      } else {
        $char = '&#9660;';
      }  
      $component->code = $char.$this->output->escapeText(' '.$component->getCaption());

      parent::renderContent($component,$flags);
      
      $this->output->rTagShort('input',array(
        'type' => 'hidden',
        'id' => $component->getId().'-expanded',
        'name' => $component->getId().'-expanded',
        'value' => ($control->visible ? '1' : '0')
      ));
      
      $component->code = $code;
       
      if($this->road->browser->supportsJavascript()) {
        $this->output->rComponentJQuery($component,'$(this).expandButton($("#'
        .$control->getId()
        .'"),'.($component->slideEffect ? 'true' : 'false').');');
      }
    }
  }
  class TXhtmlCustomButton extends TXhtmlSubmitButton {
    /**
    * @param TSubmitButton
    */
    public function renderContent(TComponent $component, $flags = null)  {
      $attributes = $this->getAttributes($component);
      $attributes['name'] = 'rAction[]';
      $attributes['type'] = 'submit';
      $attributes['value'] = $component->getId().'-click()';      
      $this->output->rTagOpen('button',$attributes);
      if(count($component->components) > 0) {
        $this->renderChildren($component,$flags);
      } else {
        $this->output->rText($component->getCaption());
      }
      $this->output->rTagClose('button');
      $this->output->rInputErrorBox($component);      
    }
  }
  class TXhtmlButton extends TXhtmlCustomButton {    
    /**
    * @param TButton
    */
    public function renderContent(TComponent $component, $flags = null)  {
      parent::renderContent($component,$flags);
      $this->renderJavaScriptEnhancements($component);
    }
  }
  class TXhtmlInsertButton extends TXhtmlCustomButton {    
    /**
    * @param TInsertButton
    */
    public function renderContent(TComponent $component, $flags = null)  {
      $pager = $component->getPager();
      $attributes = $this->getAttributes($component);
      $attributes['name'] = $pager->getId().'-insert';
      $attributes['type'] = 'submit';
      $attributes['value'] = $pager->newItemCount+$component->howMany;      
      $this->output->rTagOpen('button',$attributes);
      if(count($component->components) > 0) {
        $this->renderChildren($component,$flags);
      } else {
        $this->output->rText($component->getCaption());
      }
      $this->output->rTagClose('button');
      $this->output->rInputErrorBox($component);
    }
  }

  class TXhtmlHtmlData extends TXhtmlRenderer {
    /**
    * @param THtmlData
    */
    public function renderContent(TComponent $component, $flags = null) {
      if($component->plainText) {
        $this->output->rText($component->data);
      } else {
        $this->output->rCode($component->data);
      }
    }
  }
  /**
  *
  */
  class TXhtmlHiddenData extends TXhtmlRenderer {
    /**
    * @param THiddenData
    */
    public function renderContent(TComponent $component, $flags = null) {
      if(is_array($component->data)) {
        foreach($component->data as $key => $value) {
          if(is_bool($value)) {
            $value = ($value ? '1' : '0');
          }
          $this->output->rTagShort('input',array(
            'name' => $component->getId().'['.$key.']',
            'type' => 'hidden',
            'value' => $value
          ));
        }        
      } else {
        $value = $component->data;
        if(is_bool($value)) {
          $value = ($value ? '1' : '0');
        }

        $this->output->rTagShort('input',array(
          'name' => $component->getId(),
          'type' => 'hidden',
          'value' => $value
        ));
      }

    }
  }  
  class TXhtmlCheckBox extends TXhtmlRenderer {
    /**
    * @param TCheckBox
    */
    public function renderContent(TComponent $component, $flags = null) {
      if(!$component->placeBeforeLabel) {
        $this->renderLabel($component);
      }
      $attributes = $this->getAttributes($component,ATTRIBUTE_ALL);
      $attributes['name'] = $component->getId();
      $attributes['type'] = 'checkbox';
      if($component->isChecked()) {
        $attributes['checked'] = true;
      }
      $this->output->rTagShort('input',$attributes);
      if($component->placeBeforeLabel) {
        $this->renderLabel($component,false);
      }      
    }
  }
  class TXhtmlListerColumn extends TXhtmlRenderer {
    public function getAttributes($control, $attributeMask = ATTRIBUTE_STANDARD) {
      $attributes = parent::getAttributes($control,$attributeMask);
      $attributes['class'] .= ' '.$control->getId();
      if($control->sortOrder == SORT_ASCENDING) {
        $attributes['class'] .= ' rSortedAscending';
      } elseif($control->sortOrder == SORT_DESCENDING) {
        $attributes['class'] .= ' rSortedDescending';
      }
      if($f = $control->getFormat()) {
        $attributes['class'] .= ' '.$f->getClass();
      }
      return $attributes;
    }
    /**
    * @param TListerColumn
    */
    public function renderHeader(TComponent $component, $flags = null)  {
      $this->output->rTagOpen('th',$this->getAttributes($component, ATTRIBUTE_CLASS | ATTRIBUTE_WIDTH | ATTRIBUTE_VISIBLE | ATTRIBUTE_TITLE));
      
      if($component->options & COL_SORTABLE) {      
        $this->output->rLinkStart('?'.$component->parent->getId().'-order='.urlencode(($component->sortOrder === SORT_ASCENDING ? '-' : '').$component->getId()),
          array(
            'class'=>'rLister-sort',
            'title'=> sprintf(($component->sortOrder == SORT_ASCENDING ? LNG_SORT_DESC_BY : LNG_SORT_ASC_BY),$component->getCaption())
          )
        );
      }
      $this->output->rText($component->getCaption());
      if($component->options & COL_SORTABLE) {                                                                                                            
        $this->output->rTagClose('a');
      }
      if($component->sortOrder === SORT_ASCENDING) {
        $this->output->rTagCode('span','&#9650;',array('class'=>'rLister-order'));        
      } elseif($component->sortOrder === SORT_DESCENDING) {
        $this->output->rTagCode('span','&#9660;',array('class'=>'rLister-order'));      
      }

      $this->output->rTagClose('th');      
    }
    /**
    * @param TListerColumn
    */
    public function renderContent(TComponent $component, $flags = null)  {
      $attributes = $this->getAttributes($component, ATTRIBUTE_CLASS | ATTRIBUTE_WIDTH | ATTRIBUTE_VISIBLE | ATTRIBUTE_STYLE);      
      if($title = $component->getTitle()) {
        $attributes['title'] = $title;      
      }
      $link = $component->getHref();      
      if(is_string($link) && ($link !== '')) {
        $this->output->rTagOpen('td',$attributes);
        $this->output->rLinkText($link,$component->getText());
        $this->output->rTagClose('td');
      } else {
        $this->output->rTagText('td',$component->getText(),$attributes);
      }
    }    
    /**
    * @param TListerColumn
    */
    public function renderSortMenu(TComponent $component, $flags = null)  {
/*      if($component->options & COL_SORTABLE) {
        $this->output->rTagOpen('span',array('class'=>'rLister-sortMenu'));                  
        $this->output->rLinkCode(changeUrlQuery(absoluteUri(':'),array($component->parent->getId().'-order' => $component->getId())),LNG_ASCENDING.' &#9650;',array('class'=>'rLister-sort'));
        $this->output->rCode(' ');
        $this->output->rLinkCode(changeUrlQuery(absoluteUri(':'),array($component->parent->getId().'-order' => '-'.$component->getId())),LNG_DESCENDING.' &#9660;',array('class'=>'rLister-sort'));
        $this->output->rTagClose('span');
      }*/
    }
    /**
    * @param TListerColumn
    */
    public function renderMenu(TComponent $component, $flags = null)  {
      $this->renderSortMenu($component,$flags);
      $this->output->rCode(' ');
      if($component->options & COL_FILTERABLE) {
        $this->output->rTagText('strong',$component->getCaption().':');
        $this->output->rCode(' ');        


        if($component->filterFrom) {
          $this->output->rTagText('label',LNG_FROM.':',array('for'=>$component->getId().'-from'));
          $this->output->rCode(' ');
          $this->output->rTagShort('input',array('type'=>'text','id'=>$component->getId().'-from','name'=>$component->getId().'-from','class'=>'rEditBox','value'=>$component->filterFrom->getAsInput()));
          $this->output->rCode(' ');          
          $this->output->rTagText('label',LNG_TO.':',array('for'=>$component->getId().'-to'));
          $this->output->rCode(' ');
          $this->output->rTagShort('input',array('type'=>'text','id'=>$component->getId().'-to','name'=>$component->getId().'-to','class'=>'rEditBox','value'=>$component->filterTo->getAsInput()));
        } else {
          $this->output->rTagText('label',LNG_FIND_TEXT.':',array('for'=>$component->getId().'-filter'));
          $this->output->rCode(' ');
          $this->output->rTagShort('input',array('type'=>'text','id'=>$component->getId().'-filter','name'=>$component->getId().'-filter','class'=>'rEditBox','value'=>$component->filter));
        }
        $this->output->rCode(' ');
        $this->output->rTagShort('input',array('type'=>'submit','value'=>LNG_FILTER,'class'=>'rButton'));        
      }
    }
  }
  class TXhtmlLister extends TXhtmlCustomPager {
    /**
    * @param TLister
    */
    public function renderContent(TComponent $component, $flags = null) {
      $this->output->rInputErrorBox($component);
      $attributes = $this->getAttributes($component);
      $attributes['class'] .= ' '.$this->getNavigatorStyle($component->navigator); 
      $this->output->rTagOpen('div',$attributes);
      if($component->isFiltered()) {
        $this->data[$component->getId().'-filter'] = $component->getFilter();
        $this->output->rTagShort('input',array('type'=>'hidden','name'=>$component->getId().'-filter','value'=>$component->getFilter()));
      }
      $this->renderNavigator($component,PAGER_NAV_TOP,$flags);
      $this->renderNavigator($component,PAGER_NAV_LEFT,$flags);
      $this->renderNavigator($component,PAGER_NAV_RIGHT,$flags);
      $this->output->rTagOpen('div',array(
        'id' => $component->getId().'-content',
        'class' => 'rPager-content'
      ));
      $this->output->rTagOpen('table');
      $this->output->rTagOpen('thead');
      $this->output->rTagOpen('tr');
      foreach($component->components as /** @var TListerColumn */ $col) {
        $col->getRenderer()->renderHeader($col,$flags);
      }
      $this->output->rTagClose('tr');
/*      if($component->menuColumn !== null) {
        $this->output->rTagOpen('tr',array('class' => 'rLister-menu'));
        $this->output->rTagOpen('td',array('class' => 'rLister-menu', 'colspan'=> count($component->components)));
        $component->menuColumn->getRenderer()->renderMenu($component->menuColumn,$flags);
        $this->output->rTagClose('td');
        $this->output->rTagClose('tr');        
      }
*/
      $this->output->rTagClose('thead');
      $this->output->rTagOpen('tfoot');
      $this->output->rTagOpen('tr');
      $this->output->rTagCode('td',sprintf(LNG_DISPLAYING_ITEMS,
        number_format($component->getDisplayedCount(),0,',',' '),
        number_format($component->total,0,',',' ')),
        array('colspan' => count($component->components))
      );      
      $this->output->rTagClose('tr');      
      $this->output->rTagClose('tfoot');
      $this->output->rTagOpen('tbody');

      if($component->isDataLinked()) {
        if($component->data->isEmpty()) {
          $this->output->rTagOpen('tr',array('id'=>$component->getId().'-noitems'));
          $this->output->rTagText('td',LNG_NO_ITEMS_TO_DISPLAY,array(
            'class'=>'rLister-noitems',
            'colspan'=>count($component->components)
          ));
          $this->output->rTagClose('tr');          
        } else {
        
          $component->data->first();
          $row = 1;
          
          while($component->data->fetchFields()) {

            $this->output->rTagOpen('tr',array('id'=>$component->getId().'-items-'.$component->data->getRowId(),
              'class' => (($row % 2) > 0 ? 'rOdd' : 'rEven')
            ));
            foreach($component->components as /** @var TListerColumn */ $col) {
              $col->getRenderer()->renderContent($col,$flags);
            }
            $row ++;            
            $this->output->rTagClose('tr');
          }
        }
      } else {
        if(count($component->data) < 1) {
          $this->output->rTagOpen('tr',array('id'=>$component->getId().'-noitems'));
          $this->output->rTagText('td',LNG_NO_ITEMS_TO_DISPLAY,array(
            'class'=>'rLister-noitems',
            'colspan'=>count($component->components)            
          ));
          $this->output->rTagClose('tr');          
        } else {
          $row = 0;
          foreach(array_keys($component->data) as $key) {
            $component->row = &$component->data[$key];        
            $this->output->rTagOpen('tr',array('id'=>$component->getId().'-items-'.$key,
              'class' => (($row % 2) > 0 ? 'rOdd' : 'rEven')            
            ));
            foreach($component->components as /** @var TListerColumn */ $col) {
              $col->getRenderer()->renderContent($col,$flags);
            }
            $this->output->rTagClose('tr');
            $row ++;          
          }
        }
      }
        
              
      $this->output->rTagClose('tbody');
      $this->output->rTagClose('table');
      $this->output->rTagClose('div');
      $this->renderNavigator($component,PAGER_NAV_BOTTOM,$flags);
/*
      foreach(array_keys($component->items) as $id) {
        $this->output->rTagShort('input',array(
          'type' => 'hidden',
          'name' => $component->getId().'-items[]',
          'value' => $id
        ));
      }
*/            
      parent::renderContent($component,$flags);
      if($component->allowAjax()) {
        $this->output->rJQuery('
          $("#'.$component->getId().' thead a.rLister-sort").click(function(event) {
            event.preventDefault();
            $("#'.$component->getId().'").refresh({url: this.href, leaveOut: ["'.$component->getId().'-items"]});          
          });              
        ',true);
      }

      if($component->navigator & (PAGER_NAV_LEFT | PAGER_NAV_RIGHT)) {
        $this->output->rTagCode('div','&nbsp;',array(
          'class' => 'rClearer',
          'style' => 'clear: '.
          (($component->navigator & PAGER_NAV_LEFT) && ($component->navigator & PAGER_NAV_RIGHT)
          ? 'both'
          : ($component->navigator & PAGER_NAV_LEFT
            ? 'left'
            : 'right'
            )
          )
          .'; font-size: 1px'
        ));
      }
      $this->output->rTagClose('div');                        
    }
  }  
  class TXhtmlElement extends TXhtmlRenderer {
    /**
    * @param TStaticText
    */
    public function renderContent(TComponent $component, $flags = null) {
      $i = ($component->kind - TEXT_HEADER_1);
      if(($i >= 0) && ($i <= 5)) {
        $i++;
        $tag = 'h'.$i;
      } else {
        switch($component->kind) {
          case TEXT_INLINE:
            $tag = 'span';
            break;
          case TEXT_BLOCK:
            $tag = 'div';
            break;
          case TEXT_PARAGRAPH:
            $tag = 'p';
            break;
          case TEXT_PREFORMATED:
            $tag = 'pre';
            break;          
          default:
            return;  
        }
      }
      $this->output->rTagOpen($tag,$this->getAttributes($component));
      $this->renderChildren($component,$flags);
      $this->output->rTagClose($tag);
    }            
  }
  class TXhtmlStaticText extends TXhtmlElement {
    /**
    * @param TStaticText
    */
    public function renderChildren(TComponent $component, $flags = null) {
      $this->output->rText($component->getText());
    }            
  }  
  $this->output->type = OUTPUT_FULL;
  $this->output->registerRenderer(new TXhtmlDummyPage($this),'TPage');
  $this->output->registerRenderer(new TXhtmlForm($this),'TForm');
  $this->output->registerRenderer(new TXhtmlSubmitButton($this),'TSubmitButton');
  $this->output->registerRenderer(new TXhtmlTableLink($this),'TTableLink');
  $this->output->registerRenderer(new TXhtmlEditBox($this),'TEditBox');
  $this->output->registerRenderer(new TXhtmlTextArea($this),'TTextArea');  
  $this->output->registerRenderer(new TXhtmlSelectBox($this),'TSelectBox');
  $this->output->registerRenderer(new TXhtmlSelectBox($this),'TMultiSelectBox');  
  $this->output->registerRenderer(new TXhtmlGroupBox($this),'TGroupBox');
  $this->output->registerRenderer(new TXhtmlPanel($this),'TPanel');
  $this->output->registerRenderer(new TXhtmlToolBar($this),'TToolBar');
  $this->output->registerRenderer(new TXhtmlPanelOpener($this),'TPanelOpener');
  $this->output->registerRenderer(new TXhtmlControlPager($this),'TControlPager');
  $this->output->registerRenderer(new TXhtmlTagWrapper($this),'TTagWrapper');
  $this->output->registerRenderer(new TXhtmlLinkTag($this),'TLinkTag');
  $this->output->registerRenderer(new TXhtmlLinkTag($this),'TLinkButton');
  $this->output->registerRenderer(new TXhtmlExpandButton($this),'TExpandButton');
  $this->output->registerRenderer(new TXhtmlButton($this),'TButton');
  $this->output->registerRenderer(new TXhtmlInsertButton($this),'TInsertButton');
  $this->output->registerRenderer(new TXhtmlHtmlData($this),'THtmlData');
  $this->output->registerRenderer(new TXhtmlHiddenData($this),'THiddenData');
  $this->output->registerRenderer(new TXhtmlCheckBox($this),'TCheckBox');
  $this->output->registerRenderer(new TXhtmlLister($this),'TLister');
  $this->output->registerRenderer(new TXhtmlListerColumn($this),'TListerColumn');
  $this->output->registerRenderer(new TXhtmlStaticText($this),'TStaticText');
?>