<?php
  class TJsonOutput extends TCustomOutput {
//    public $contentType = 'application/json';  
    public $contentType = 'text/plain';
    public function rCode($buffer) {
      $this->write($buffer);
    }
    public function ajaxRedirect($url)  {
      $obj = new stdClass();
      $obj->redirect = $url;
      $this->rValue($obj);
      return true;
    }
    public function warningReported($warning, $warningCount)  {

      return true;
    }
    
    public function renderError($errno, $errstr, $errfile = '', $errline = 0, $errcontext = array()) {
      $obj = new stdClass();
      $obj->error = new stdClass();
      $obj->error->message = strip_tags($errstr);
      $obj->error->code = $errno;
      $obj->error->file = $errfile;
      $obj->error->line = $errline;
      $this->rValue($obj);
      exit;
    }
    /**
    *
    *
    * @return void
    */
    public function fetchMessages($obj)  {
      $obj->messages = new stdClass();
      $obj->messages->success = $this->road->dequeueMessages(MSG_SUCCESS);
      $obj->messages->error = $this->road->dequeueMessages(MSG_ERROR);
      $obj->messages->highlight = $this->road->dequeueMessages(MSG_HIGHLIGHT);
      $this->road->clearMessages();    
    }
    /**
    * 
    *
    * @param Exception
    * @return bool
    */
    public function renderException($exception) {
      $this->renderError($exception->getCode(),
        $exception->getMessage(),
        $exception->getFile(),
        $exception->getLine()
      );
    }        
/*    public function rSimpleValue($value) {
      $this->write(json_encode($value));
    }*/ 
    public function rValue($value) {
/*      if(is_object($value)) {
        $this->rObject($value);
      } elseif(is_array($value)) {
        $this->rArray($value);
      } else { 
        $this->rSimpleValue($value);
      }*/
      $this->write(json_encode($value));
    }
/*    public function rPair($name, $value) {
      $this->rSimpleValue($name);
      $this->rCode(' : ');
      $this->rValue($value);
    }
    public function rObject($obj) {
      $notFirst = false;
      $this->rCode('{ ');
      foreach($obj as $var => $val) {
        if($notFirst) {
          $this->rCode(', ');
        }      
        $this->rPair($var,$val);
        $notFirst = true;        
      }
      $this->rCode('} ');    
    }
    public function rArray($arr) {
      $notFirst = false;
      $this->rCode('[ ');
      foreach($arr as $val) {
        if($notFirst) {
          $this->rCode(', ');
        }
        $this->rValue($val);
        $notFirst = true;        
      }
      $this->rCode(' ]');      
    }*/
  }
  class TJsonRenderer extends TCustomRenderer {
    /** @var TJsonOutput */
    public $output;
    /**
    * Add data
    *
    * @return void
    */
    public function addData(TComponent $component, &$data, $flags = null) {}
    public function render(TComponent $component, $flags = null) {
      $data = new stdClass();      
      if($component->javaScript) {
//      if(!$flags && $component->javaScript) {
        $data->javaScript = $component->javaScript;
      }
      if($component->jQuery) {
        if(!isset($data->javaScript)) {
          $data->javaScript = ''; 
        }
        $data->javaScript .=  ' $("#'.$component->getId().'").each(function() {'.$component->jQuery.'});';
      }
      $this->addData($component,$data,$flags);
      $this->output->rValue($data);
    }    
  }
  class TJsonSelectBox extends TJsonRenderer {
    /**
    * @param TSelectBox
    */
    public function addData(TComponent $component, &$data, $flags = null) {
      if($flags == 'options') {
        $data = mergeObjects($component->toJson($flags));
      } else {
        parent::addData($component, $data, $flags);
      }    
    }
  }
  class TJsonEditBox extends TJsonRenderer {
    /**
    * @param TEditBox
    */
    public function addData(TComponent $component, &$data, $flags = null) {
      if($flags == 'autoComplete') {
        $data = mergeObjects($component->toJson($flags));        
      } else {
        parent::addData($component, $data, $flags);
      }    
    }    
  }
  /**
  *
  */
  class TJsonPage extends TJsonRenderer {
    private $headerRendered = false;
    private $footerRendered = false;
    /**
    * @param TPage Page
    */
    public function renderHeader(TComponent $component)  {
      if($this->headerRendered) {
        return;
      } else {
        $this->headerRendered = true;
      }

    }
  }
  class TJsonSubmitButton extends TJsonRenderer {
    private $errors = array();
    /**
    * @param TComponent
    */
    public function addErrors($component) {
      if($msg = $component->getErrorMessage()) {
        $obj = new stdClass();
        $obj->name = $component->getId();
        $obj->message = $msg;  
        $this->errors[] = $obj;
      }
      if($component->isInstanceOf('TControlPager')) {
        foreach($component->items as $components) {
          foreach($components as $c) {
            $this->addErrors($c);
          }
        }
      } else {
        foreach($component->components as $c) {
          $this->addErrors($c);
        }
      }      

    }
    /**
    * @param TDBLink
    */
    public function addData(TComponent $component, &$data, $flags = null) {
      if($flags == 'dynamicSubmit') {
        $this->output->fetchMessages($data);
        if(method_exists($component,'wasSuccessful')) {
          $data->success = $component->wasSuccessful();
        }
        if($component->form->isErrorneous()) {
          $this->addErrors($component->form);
          $data->inputErrors = $this->errors;
        }
      } else {
        parent::addData($component, $data, $flags);
      }            
    }    
  }  
  $this->output = new TJsonOutput($this);
  $this->output->type = OUTPUT_JSON;  
  $this->output->registerRenderer(new TJsonEditBox($this),'TEditBox');
  $this->output->registerRenderer(new TJsonSelectBox($this),'TSelectBox');
  $this->output->registerRenderer(new TJsonSelectBox($this),'TMultiSelectBox');
  $this->output->registerRenderer(new TJsonSubmitButton($this),'TSubmitButton');      
?>