<?php
  define('ATTRIBUTE_STANDARD',ATTRIBUTE_ALL | !ATTRIBUTE_WIDTH | !ATTRIBUTE_DISABLED | !ATTRIBUTE_READONLY);
  require_once(CFG_ROAD_DIR.'/output/full/base.php');
  /**
  *
  */
  abstract class TXhtmlRenderer extends TCustomXhtmlRenderer {
    protected function getWidth($control) {
      return false;      
    }    
    protected function getHeight($control) {
      return $this->checkUnits($control->height);      
    }
  }

  require_once(CFG_ROAD_DIR.'/output/full/common.php');  

  /**
  *
  */
  class TMobilePage extends TXhtmlPage {
    public function renderHeader(TComponent $component, $flags = null)  {
      if($this->headerRendered) return;

      if($this->road->browser->supportsJavaScript()) {
        $this->metaAddCustom('<script type="text/javascript">var mobileVersion = true;</script>');
      }
      parent::renderHeader($component,$flags);
    }
    public function beforeRender(TComponent $component, $flags = null)  {
//      parent::beforeRender($component,$flags);
      $this->metaAddScript('/road/core/road.js');
      $this->metaAddCustom('<meta name="HandheldFriendly" content="true"/>');
      $this->metaAddCustom('<meta name="viewport" content="width=device-width, initial-scale=1.0"/>');
      $this->metaAddStyle('/'.relativePath(dirname(__FILE__)).'/style.css');
    }
  }

  /**
  *
  */
  class TMobileEditBox extends TXhtmlRenderer {    
    /**
    * @param TEditBox
    */
    public function renderContent(TComponent $component, $flags = null)  {

      if($component->displayCaption) {      
        $this->renderLabel($component);
      }
      $attributes = $this->getAttributes($component,ATTRIBUTE_ALL);
      if($format = $component->getFormat()) {
        $attributes['class'] .= ' '.$format->getClass();        
      }

      if($component->notBlank) {
        $attributes['class'] .= ' rNotBlank';      
      }
      $attributes['name'] = $component->name;
      if($component->passwordInput) {
        $attributes['type'] = 'password';
      } else {
        $attributes['type'] = 'text';
      }
      if(!$component->passwordInput || $component->keepPassword) {
        $attributes['value'] = $component->value;
      }
      if(!$component->browserAutoComplete && !$this->road->browser->supportsJavaScript()) {
        // we have to use xhtml invalid autocomplete attribute
        $attributes['autocomplete'] = 'off';        
      }
      $this->output->rTagShort('input',$attributes);

      $this->output->rInputErrorBox($component);
      
      if($this->road->browser->supportsJavaScript()) {
        if(!$component->browserAutoComplete) {
          $this->output->rComponentJQuery($component,'$(this).attr("autocomplete","off");');
        }      
        if($component->autoComplete->enabled) {
          if(!$component->dynamic) {
            $component->fetchList();
            if(count($component->autoCompleteList) < 1) {
              return;
            }
          }
          $options = $component->autoComplete->toJson();
          if($component->dynamic) {
            $options->dynamic = true;
          } else {
            $options->list = $component->toJson('autoComplete'); 
          }
          $this->output->rComponentJQuery($component,'$(this).autocompletion('.json_encode($options).');');             
        }
      }
    } 
  }


  class TMobileTextArea extends TXhtmlRenderer {    
    /**
    * @param TTextArea
    */
    public function renderContent(TComponent $component, $flags = null)  {
      if($component->displayCaption) {      
        $this->renderLabel($component);
      }
      $attributes = $this->getAttributes($component,ATTRIBUTE_ALL);
      if($format = $component->getFormat()) {
        if(isset($attributes['class'])) {
          $attributes['class'] .= ' '.$format->getClass();
        } else {
          $attributes['class'] .= $format->getClass();
        }        
      }

      $attributes['name'] = $component->name;
      $attributes['rows'] = $component->rows;
//      $attributes['cols'] = $component->cols;      

      $this->output->rTagText('textarea',$component->value,$attributes);

      $this->output->rInputErrorBox($component);

    } 
  }
  
  class TMobileSelectBox extends TXhtmlRenderer {

    /**
    * @param TSelectOption
    * @param TSelectBox
    */        
    private function renderOption($option, $select, $type = SELECTBOX_STANDARD) {
      $attributes = $this->getAttributes($option,ATTRIBUTE_ALL);
      if($option->isSelected()) {
        $attributes['selected'] = true;
      }
      $attributes['value'] = $option->value;
      $this->output->rTagText('option',$option->getCaption(),$attributes);              
    }
    /** @param TSelectBox */
    private function renderOptions($component, $type = SELECTBOX_STANDARD) {
      if(count($component->groups) > 0) {
        $groups = array();
        foreach($component->options as /** @var TSelectOption */ $option) {        
/*          if(($option->value == null) && $component->isRequired()) {
            continue;
          }*/
          $grp = $option->group;
          if($grp !== null) {
            $found = false;
            foreach($component->groups as $group) {
              if($group->value == $grp) {
                $found = true;
                break;              
              }
            }
//            if(!isset($component->groups[$group])) {          
            if(!$found) {
              triggerError(E_SELECT_BOX_GROUP_NOT_DEFINED,array($grp,$component->name),E_USER_WARNING);
              $grp = null;
            }
          }           
          $groups[$grp][] = $option;
        }      
        if(isset($groups[null])) {
          foreach($groups[null] as $option) {
            $this->renderOption($option,$component,$type);
          }
        }
        
        foreach($component->groups as $group) {
          $gi = $group->value;
          if(!($component->showEmptyGroups || isset($groups[$gi]))) {
            continue;
          }
          $attributes = $this->getAttributes($group);
          $attributes['label'] = $group->getCaption();  
          $this->output->rTagOpen('optgroup',$attributes); 
          if(isset($groups[$gi])) {
            foreach($groups[$gi] as $option) {
              $this->renderOption($option,$component,$type);
            }
          }
          $this->output->rTagClose('optgroup');
        }        
      } else {
        foreach($component->options as $option) {
          $this->renderOption($option,$component,$type);
        }      
      }
    }    
    /**
    * @param TSelectBox
    */
    public function renderContent(TComponent $component, $flags = null)  {

        $attributes = $this->getAttributes($component,ATTRIBUTE_ALL);
        if($format = $component->getFormat()) {
          if(isset($attributes['class'])) {
            $attributes['class'] .= ' '.$format->getClass();
          } else {
            $attributes['class'] = $format->getClass();
          }        
        }
        if($component->rows > 1) {
//          $attributes['size'] = 1; //$component->rows;
        }
 
        if($component->isMultiSelect()) {        
          $attributes['multiple'] = true;        
          $attributes['name'] = $component->name.'[]';
        } else {      
          $attributes['name'] = $component->name;
        }
        if($component->displayCaption) {
          $this->renderLabel($component);
        }
        $this->output->rTagOpen('select',$attributes);
        $this->renderOptions($component,SELECTBOX_STANDARD);      
  
        $this->output->rTagClose('select');

      $this->output->rInputErrorBox($component);

      if($this->road->browser->supportsJavaScript()) {

        if($component->displayStyle & SELECTBOX_COMBOBOX) {

          if(
            ($component->rows > 1)
            ||
            $component->isMultiSelect()            
          ) {
        
          } else {
            $this->output->rComponentJQuery($component,'$(this).combobox();');
          }
        } elseif($component->displayStyle & SELECTBOX_LINKER_MOBILE) {
            $this->output->rComponentJQuery($component,'$(this).linker({dynamic: '.($component->dynamic ? 'true' : 'false')
              .', emptyMessage:'.json_encode(LNG_LINKER_NOTHING_SELECTED).'});'
            );
        }
      }
    }
 
  }
 

  class TMobilePanelOpener extends TXhtmlRenderer {
    /**
    * @param TPanelOpener
    */
    public function renderContent(TComponent $component, $flags = null)  {
      if( /** @var TPanel */ $panel = $component->page->findComponent($component->panelName,true)) {
        $attibutes = $this->getAttributes($component);
        $this->output->rLinkStart('?rAction[]='.$panel->getId().'-show('.($component->exclusive ? '1':'').')'
          ,$attibutes      
        );
        $this->output->rText($component->getCaption());
        $this->output->rTagClose('a');
/*        
        if($component->allowAjax()) {
          $options = array(
            'autoScale: false',
            'scroll: "auto"',
            'showNavArrows: false',
            'ajax: {type: "POST",data: fields}'
          );
          if($component->windowHeight || $component->windowWidth) {
            $options[] = 'autoDimensions: false';
            if($component->windowWidth) {          
              $options[] = 'width: '.$component->windowWidth;
            }
            if($component->windowHeight) {
              $options[] = 'height: '.$component->windowHeight;
            }
          }
          
          $this->output->rJQuery('
            var self = $( "#'.$component->getId().'" ); 
            var form = self.parents("form:first");
            if(form.length > 0) {
              var fields = form.serializeArray();
            } else {
              var fields = new Array();
            }
            fields.push({
              name: "rAction[]",
              value: "'.$panel->getId().'-show('.($component->exclusive ? '1':'').')"
            });
            fields.push({
              name: "rAction[]",
              value: "'.$panel->getId().'-render()"
            });
            self.fancybox({
              '.implode(',',$options).'
            });
          ');        
        }*/        
      } else {
        triggerError(E_COMPONENT_NOT_FOUND,array($component->panelName),E_USER_WARNING);
      }      
    }
  }


  class TMobileToolBar extends TXhtmlRenderer {
    /**
    * @param TGroupBox
    */
    public function renderHeader(TComponent $component)  {
      $attributes = $this->getAttributes($component);
      $this->output->rTagOpen('div',$attributes);
      if($component->displayCaption) {
        $this->output->rTagText('div',$component->getCaption(),array('class' => 'rToolBarCaption'));
      }
    }
        
    /**
    * @param TGroupBox
    */    
    public function renderFooter(TComponent $component)  {
      $this->output->rTagClose('div');
    } 
  }



  class TMobileListerColumn extends TXhtmlListerColumn {
    /**
    * @param TListerColumn
    */
    public function renderHeader(TComponent $component, $flags = null)  {
      $this->output->rTagOpen('th',$this->getAttributes($component, ATTRIBUTE_CLASS | ATTRIBUTE_WIDTH | ATTRIBUTE_VISIBLE | ATTRIBUTE_TITLE));
      
      if($component->options & COL_SORTABLE) {      
        $this->output->rLinkStart('?'.$component->parent->getId().'-order='.urlencode(($component->sortOrder === SORT_ASCENDING ? '-' : '').$component->getId()),
          array(
            'class'=>'rLister-sort',
            'title'=> sprintf(($component->sortOrder == SORT_ASCENDING ? LNG_SORT_DESC_BY : LNG_SORT_ASC_BY),$component->getCaption())
          )
        );
      }
      $this->output->rText($component->getCaption());

      if($component->sortOrder === SORT_ASCENDING) {
        $this->output->rTagCode('span','&#9650;',array('class'=>'rLister-order'));        
      } elseif($component->sortOrder === SORT_DESCENDING) {
        $this->output->rTagCode('span','&#9660;',array('class'=>'rLister-order'));      
      }

      if($component->options & COL_SORTABLE) {                                                                                                            
        $this->output->rTagClose('a');
      }

      $this->output->rTagClose('th');      
    }

  }
/*  class TMobileCustomPager extends TXhtmlCustomPager {
    protected function rPageLink($pager,$code,$page,$title)  {
      echo 'whaaaaat!!!!';
      if($code == '&#9668;') {
        $code = '&lt;';
      } elseif($code == '&#9658;') {
        $code = '&gt;';
      }
      parent::rPageLink($pager,$code,$page,$title);
    }
  }*/    
  $this->output->type = OUTPUT_MOBILE;
  $this->output->registerRenderer(new TMobilePage($this),'TPage');
  $this->output->registerRenderer(new TMobileEditBox($this),'TEditBox');
  $this->output->registerRenderer(new TMobileTextArea($this),'TTextArea');  
  $this->output->registerRenderer(new TMobileSelectBox($this),'TSelectBox');
  $this->output->registerRenderer(new TMobileSelectBox($this),'TMultiSelectBox');    
//  $this->output->registerRenderer(new TMobileControlPager($this),'TControlPager');
  $this->output->registerRenderer(new TMobilePanelOpener($this),'TPanelOpener');
  $this->output->registerRenderer(new TMobileToolBar($this),'TToolBar');
  $this->output->registerRenderer(new TMobileListerColumn($this),'TListerColumn');
//  $this->output->registerRenderer(new TMobileCustomPager($this),'TCustomPager');
?>