<?php          

  /**
  * Unicode enabled case conversion 
  *
  * @param string String 1
  * @param string String 2
  * @return string
  */
  function lowerCase($string) {
    if(CFG_UTF8) {
      return mb_strtolower($string);       
    } else {
      return strtolower($string);
    }
  }
  
  /**
  * Unicode enabled case conversion 
  *
  * @param string String 1
  * @param string String 2
  * @return string
  */
  function upperCase($string) {
    if(CFG_UTF8) {
      return mb_strtoupper($string);       
    } else {
      return strtoupper($string);
    }
  }
               
  /**
  * Case sensitive string comparsion 
  *
  * @param string String 1
  * @param string String 2
  * @return bool True if both strings contains the same text
  */  
  function sameStr($str1, $str2) {
    return (strcmp($str1,$str2) === 0);
  }

  /**
  * Case in-sensitive string comparsion 
  *
  * @param string String 1
  * @param string String 2
  * @return bool True if both strings contains the same text
  */  
  function sameText($str1, $str2) {
    return sameStr(lowerCase($str1),lowerCase($str2));
  }

  /**
  * Case sensitive right substring comparsion 
  *
  * @param string Substring
  * @param string String
  * @return bool True if String ends with Substring
  */
  function endsStr($substr, $str) {
    return (strcmp(substr($str,-strlen($substr)),$substr) === 0);
  }
  /**
  * Case in-sensitive right substring comparsion 
  *
  * @param string Substring
  * @param string String
  * @return bool True if String ends with Substring
  */
  function endsText($substr, $str) {
    return endsStr(lowerCase($substr),lowerCase($str));
  }

  /**
  * Case sensitive left substring comparsion 
  *
  * @param string Substring
  * @param string String
  * @return bool True if String starts with Substring
  */
  function startsStr($substr, $str) {
    return (strcmp(substr($str,0,strlen($substr)),$substr) === 0);
  }

  /**
  * Case in-sensitive left substring comparsion 
  *
  * @param string Substring
  * @param string String
  * @return bool True if String starts with Substring
  */
  function startsText($substr, $str) {
    return startsStr(lowerCase($substr),lowerCase($str));
  }

  /**
  * Case sensitive substring comparsion 
  *
  * @param string Substring
  * @param string String
  * @return bool True if String contains Substring
  */
  function containsStr($substr, $str) {
    return (strpos($str,$substr) !== false);
  }

  /**
  * Case in-sensitive substring comparsion 
  *
  * @param string Substring
  * @param string String
  * @return bool True if String contains Substring
  */
  function containsText($substr, $str) {
    return containsStr(lowerCase($substr),lowerCase($str));
  }

  /**
  * Returns string representation of bool value 
  *
  * @param bool Bool value
  * @return string "True" for True and "False" for False
  */
  function boolToStr($value) {
    return ($value ? 'True' : 'False');
  }

  /**
  * Remove CR and LF characters 
  *
  * @param string Text
  * @return string
  */
  function stripNL($str) {
    return str_replace( array("\n","\r"), '', $str);
  }

  /**
  * Interlaces string with selected padding
  * 
  * @param string Original string
  * @param string Padding
  * @param int Interlace after every X characters  
  * @return string
  */
  function  strInterlace($str, $padding, $count = 1)  {
    $res = '';
    while(gStrLen($str) > 0) {
      $res .= gSubStr($str,0,$count);
      $res .= $padding;
      $str = gDelete($str,0,$count);            
    }
    return $res;  
  }

  /**
  * Unicode enabled length of the string 
  *
  * @param string Text
  * @return int
  */
  function strLength($str) {
    if(CFG_UTF8) {
      return iconv_strlen($str,CFG_ENCODING);
    } else {
      return strlen($str);
    }    
  }

  /**
  * Unicode enabled substring extraction 
  *
  * @param string Text
  * @param int Starting position  
  * @param int Substring length  
  * @return string
  */
  function strCopy($str,$start,$length = null) {
    if(CFG_UTF8) {
      if($length === null) {
        return mb_substr($str,$start);
      } else {
        return mb_substr($str,$start,$length);
      }

    } else {
      if($length === null) {
        return substr($str,$start);
      } else {
        return substr($str,$start,$length);      
      }        
    }
  }

  /**
  * Unicode enabled substring replacement 
  *
  * @param string Original Text
  * @param string Replacement  
  * @param int Starting position  
  * @param int Substring length  
  * @return string
  */
  function subStrReplace($string, $replacement, $start, $length = null) {
    if(defined('CFG_UTF8') && CFG_UTF8) {
      if($length === null) {
        $length = 0;//mb_strlen($replacement);
      }
      return mb_substr($string,0,$start).$replacement.mb_substr($string,$start+$length);       
    } else {
      if($length === null) {
        return substr_replace($string,$start);
      } else {
        return substr_replace($string,$start,$length);
      }
    }
  }


  /**
  * Remove characters from String 
  *
  * @param string Original string
  * @param int Index of first character to be deleted
  * @param int How many characters should be deleted
  * @return string Modified string
  */
  function strDelete($str, $start, $length = null) {
    if($length === null) {
      return subStrReplace($str, '', $start);
    } else {
      return subStrReplace($str, '', $start, $length);
    }
  }


  /**
  * Convert first character to upper case 
  *
  * @param string Original Text  
  * @return string
  */
  function capitalize($string) {
    if(CFG_UTF8) {
      return mb_strtoupper(
        mb_substr($string,0,1)        
        ).mb_substr($string,1);  
    } else {
      return strtoupper(
        substr($string,0,1)        
        ).substr($string,1);                  
    }
  }  

  /**
  * Convert first character to upper case 
  *
  * @param string Original text
  * @param int Max length of the intro string
  * @return string
  */
  function strIntro($string, $max_length) {
    $len = strLength($string);
    if($len <= $max_length) {
      return $string;
    }
    return strCopy($string,0,$max_length-3).'...';
  }

  /**
  * Convert Unicode or Ansi string to ASCII  
  *
  * @param string Text
  * @return string
  */
  function strToASCII($value) {
    if(CFG_WINDOWS) {
      return str_replace("'",'',iconv(CFG_ENCODING, "us-ascii//TRANSLIT", $value));    
    } else {
      return iconv(CFG_ENCODING, "us-ascii//TRANSLIT", $value);
    }
  }
  
  /**
  * Convert text to URL suitable form  
  *
  * @param string Text
  * @return string
  */  
  function urlFriendly($value) {
    $value = strtolower(strToASCII($value));
    
    $l = strlen($value);
    $res = '';
    $sep = false;
    for($i = 0; $i < $l; $i++) {    
      $x = ord($value[$i]);    
      if((($x >= 48) && ($x <= 57)) || (($x >= 97) && ($x <= 122))) {
        $res .= $value[$i];
        $sep = false; 
      } else {
        if(!$sep) {
          $sep = true;
          if (($i < ($l-1)) && ($i > 0)) {
            $res .= '-';
          }
        }      
      }       
    }
    return $res;
  }
    
  /**
  * Alias to HTMLspecialchars
  * @param string Text to encode
  * @return string
  */
  function h($text)  {
    return htmlspecialchars($text);  
  }

  /**
  * Finds out if iconv supports chosen encoding
  *
  * @param string encoding
  * @return bool
  */  
  function encodingSupported($encoding) {
    if(iconv('utf-8',$encoding,'b') === false) {
      return false;      
    } else {
      return true;    
    }
  }
  function includeTrailingSlash($path) {
    if(!endsStr('/',$path)) {
      $path .= '/';
    }
    return $path;
  }
  function stripTrailingSlash($path) {
    if(endsStr('/',$path)) {
      $path = strDelete($path,strLength($path)-1,1);
    }
    return $path;
  }  

  function relativeUri($path) {
    if($path == ':') {
      $path = substr($_SERVER['REQUEST_URI'],strlen(CFG_WEB_ROOT));
    } else {
      if(substr($path,0,1) == '.') {      
        $path = CFG_SCRIPT_URL.substr($path,1);
      } elseif(substr($path,0,1) == '?') {
        $path = changeUrlQuery(substr($_SERVER['REQUEST_URI'],strlen(CFG_WEB_ROOT)),substr($path,1));
      } elseif(substr($path,0,1) == '/') {
        $path = $path;
      } else {
        $url = parse_url($path);
        if(isset($url['scheme'])) {
          return false;
        } else {
          $x = filePath(CFG_SCRIPT_URL);
          if($x == '.') {
            $x = '';
          }
          $path = includeTrailingSlash($x).$path;
        }
      }
    }
    global $road;
    if(isset($road) && ($road->browser !== null)) {
      if(!$road->browser->supportsCookies()) {
        if(CFG_ALLOW_URL_SESSION && ($id = session_id()) && ($id <> 'nofuckingsessionid')) {
          $path = changeUrlQuery($path,array(session_name()=>$id, 'rBrowseCaps'=>false));
        } elseif($road->browser->browserDetected) {
          $path = changeUrlQuery($path,array(
            session_name()=> false,            
            'rBrowserCaps'=>
            ($road->browser->supportsJavascript() ? '1' : '0')
            .($road->browser->supportsAjax() ? '1' : '0').'0'          
          ));
        }
      }
    }
    return $path;
  }


  function absoluteUri($path, $includeSite = true) {
  
    $x = relativeUri($path);
    
    if($x === false) {
      return $path;
    } else {    
      $path = CFG_WEB_ROOT.$x;    
    } 
/*    if($path == ':') {
      $path = $_SERVER['REQUEST_URI'];
    } else {
      if(substr($path,0,1) == '.') {      
        $path = CFG_SCRIPT_URL.substr($path,1);
      } elseif(substr($path,0,1) == '?') {
        $path = CFG_SCRIPT_URL.substr($path,1);
      } elseif(substr($path,0,1) == '/') {
        $path = stripTrailingSlash(CFG_WEB_ROOT).$path;
      } else {
        $url = parse_url($path);
        if(isset($url['scheme'])) {
          return $path;
        } else {
          $path = includeTrailingSlash(stripTrailingSlash(CFG_WEB_ROOT).filePath(CFG_SCRIPT_URL)).$path;
        }
      }
    }
    global $road;
    if(isset($road) && ($road->browser !== null)) {
      if(!$road->browser->supportsCookies()) {
        if(CFG_ALLOW_URL_SESSION && ($id = session_id()) && ($id <> 'nofuckingsessionid')) {
          $path = changeUrlQuery($path,array(session_name()=>$id, 'rBrowseCaps'=>false));
        } elseif($road->browser->browserDetected) {
          $path = changeUrlQuery($path,array(
            session_name()=> false,            
            'rBrowserCaps'=>
            ($road->browser->supportsJavascript() ? '1' : '0')
            .($road->browser->supportsAjax() ? '1' : '0').'0'          
          ));
        }
      }
    }*/
    if($includeSite) {
      return stripTrailingSlash(CFG_SITE).$path;
    } else {
      return $path;    
    }
  }
  

  if(!function_exists('strptime')) {

    // windows version of PHP misses the strptime function,
    // so we have to implement some pieces of it
    function strptime($str,$format) {
    
      // at we need to convert format for sscanf function
      $scanFormat = '';
      $scanEquivs = array();
      $len = strlen($format);
      $i = 0;
      $perc = false;
      while ($i < $len) {
        $char = substr($format,$i,1); 
        if($perc) {
          $char = substr($format,$i,1);            
          switch($char) {
//              case 'c':        // - preferred date and time representation for the current locale
            case 'n':        // - newline character
              $scanFormat .= "\n"; 
              break;
            case 't':        // - tab character 
              $scanFormat .= "\t";
              break;
            case '%':        // - a literal `%' character              
              $scanFormat .= "%";
              break;                   
            case 'd':        // - day of the month as a decimal number (range 01 to 31)
              $scanFormat .= '%2d';
              $scanEquivs[] = $char;
              break;
            case 'z':        // - same as %m/%d/%y
              $scanFormat .= '%5s';
              $scanEquivs[] = 'z';                                
              break;
            case 'e':        // - day of the month as a decimal number, a single digit is preceded by a space (range ' 1' to '31')
              $scanFormat .= '%2d';
              $scanEquivs[] = $char;
              break;
            case 'H':        // - hour as a decimal number using a 24-hour clock (range 00 to 23)
              $scanFormat .= '%2d';
              $scanEquivs[] = $char;
              break;
            case 'I':        // - hour as a decimal number using a 12-hour clock (range 01 to 12)
              $scanFormat .= '%2d';
              $scanEquivs[] = $char;
              break;
            case 'j':        // - day of the year as a decimal number (range 001 to 366) 
              $scanFormat .= '%3d';
              $scanEquivs[] = $char;
              break;
            case 'm':        // - month as a decimal number (range 01 to 12) 
              $scanFormat .= '%2d';
              $scanEquivs[] = $char;
              break;
            case 'M':        // - minute as a decimal number 
              $scanFormat .= '%2d';
              $scanEquivs[] = $char;
              break; 
            case 'p':        // - either `am' or `pm' according to the given time value, or the corresponding strings for the current locale  
              $scanFormat .= '%2s';
              $scanEquivs[] = $char;
              break;
            case 'S':        // - second as a decimal number 
              $scanFormat .= '%2d';
              $scanEquivs[] = $char;
              break; 
            case 'T':        // - current time, equal to %H:%M:%S 
              $scanFormat .= '%2d:%2d:%2d';
              $scanEquivs[] = 'H';
              $scanEquivs[] = 'M';
              $scanEquivs[] = 'S';
              break;
            case 'D':        // - current time, equal to %H:%M:%S 
              $scanFormat .= '%2d/%2d/%2d';
              $scanEquivs[] = 'm';
              $scanEquivs[] = 'd';
              $scanEquivs[] = 'y';
              break;
//              case 'x':        // - preferred date representation for the current locale without the time 
//              case 'X':        // - preferred time representation for the current locale without the date 
            case 'y':        // - year as a decimal number without a century (range 00 to 99) 
              $scanFormat .= '%2d';
              $scanEquivs[] = $char;
              break;
            case 'u':        // - year as a decimal number without a century (range 00 to 99) 
              $scanFormat .= '%1d';
              $scanEquivs[] = $char;
              break;              
            case 'Y':        // - year as a decimal number including the century
              $scanFormat .= '%4d';
              $scanEquivs[] = $char;
              break;
            default:
              if(error_reporting() >= E_WARNING) {
                echo '<br/><b>Warning:</b> "%'.$char.'" format not supported by strptime windows replacement<br/>'."\n";
              }
              break;
          }
          $perc = false;
        } else {
          if($char == '%') {         
            $perc = true;
          } else {
          $scanFormat .= $char;
          }        
        }
        $i++;
      }

      //now we parse the string with sscanf
      $res = sscanf($str,$scanFormat);

      if(($res === -1) || ($res === false) || ($res === null)) {
        return false;
      }
      
      //prepare result array 
      $result = array(
        'tm_sec' => 0,
        'tm_min' => 0,        
        'tm_hour' => 0,
        'tm_mday' => null,
        'tm_mon' => null,        
        'tm_year' => null,
        'tm_wday' => null,
        'unparsed' => ''
      );
      $pm = false;
      $tz = 0;
      $i = -1;            
      //and use the result field whatever they mean
      foreach($res as $i => $value) {
        $char = $scanEquivs[$i];
        if(($char != 'S') && ($value === null)) return false;
        switch($char) {                   
          case 'd':        // - day of the month as a decimal number (range 01 to 31)
            $result['tm_mday'] = $value;
            break;
          case 'e':        // - day of the month as a decimal number, a single digit is preceded by a space (range ' 1' to '31')
            $result['tm_mday'] = $value;
            break;
          case 'H':        // - hour as a decimal number using a 24-hour clock (range 00 to 23)
            $result['tm_hour'] = $value;
            break;
          case 'I':        // - hour as a decimal number using a 12-hour clock (range 01 to 12)
            $result['tm_hour'] = $value;
            break;
          case 'j':        // - day of the year as a decimal number (range 001 to 366) 
            $result['tm_yday'] = $value;
            break;
          case 'm':        // - month as a decimal number (range 01 to 12) 
            $result['tm_mon'] = $value -1;
            break;
          case 'M':        // - minute as a decimal number 
            $result['tm_min'] = $value;
            break; 
          case 'p':        // - either `am' or `pm' according to the given time value, or the corresponding strings for the current locale  
            if($value == 'pm') {
              $pm = true;              
            }
            break;
          case 'S':        // - second as a decimal number 
            $result['tm_sec'] = $value;
            break;
          case 'z':        // - second as a decimal number
            $tz = str_replace(':','',$value);
            $tz = date('O') - $tz;
            echo 'shit: '.$tz; 
            break;
          case 'u':
            if($value === null) {
              $result['tm_wday'] = null;
            } else {          
              $result['tm_wday'] = $value - 1;
            }
            break;
          case 'y':        // - year as a decimal number without a century (range 00 to 99)
            if($value === null) {
              $result['tm_year'] = null;
            } else {
              if($value >= 90) { 
                $result['tm_year'] = $value;
              } else {
                $result['tm_year'] = $value + 100;            
              }
            }
            break;
          case 'Y':        // - year as a decimal number including the century
            if($value === null) {
              $result['tm_year'] = null;
            } else {            
              $result['tm_year'] = $value - 1900;
            }
            break;
          default:
            break;
        }
      }
/*      $i++;
      $result['unparsed'] = $res[$i];*/

      if($result['tm_hour'] !== null) {
        if($pm) {
          $result['tm_hour'] += 12;
        }
        $result['tm_hour'] += $tz/100;
      }

      return $result;
    }
  }

  function filePath($filename) {
    $res = dirname($filename);
    if(CFG_WINDOWS) {
      $res = str_replace('\\','/',$res);
    }
    return $res;
  }
  function fileExtension($filename) {
    $path_info = pathinfo($filename);
    if(isset($path_info['extension'])) {
      return $path_info['extension'];
    } else {
      return '';
    }
  }
  /**
  * Returns path relative to web-root
  *
  * @return mixed Relative path or false if $filename is not inside web-root
  */
  function relativePath($filename) {
    $filename = realpath($filename);
    if(CFG_WINDOWS) {
      $filename = str_replace('\\','/',$filename);
    }
    if(startsStr(CFG_ROOT_DIR,$filename)) {
      return $filename = strDelete($filename,0,strLength(CFG_ROOT_DIR)+1);
    } else {
      return false;
    }
  }
  $__roadSqlKeyWords = array(
    'SELECT' => false,
    'INTO' => false,
    'FROM' => false,

    'INSERT' => true,    
    'INSERT INTO' => false,
    'VALUES' => false,

    'UPDATE' => false,
    'SET' => false,

    'WHERE' => false,
    'GROUP' => true,
    'GROUP BY' => false,
    'HAVING' => false,
    'ORDER' => true,
    'ORDER BY' => false,
    'LIMIT' => false,
    'OFFSET' => false
  );
  function sqlExplode($query) {
    global $__roadSqlKeyWords;
    $string = false;
    $escaped = false;
    $field = false;
    $level = 0;
    $topLevel = 0;
    $queryLength = strlen($query);
    $identifierChars = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
    $result = array();
    $sectionCount = 0;
    $lastSection = 0;
    $result[0] = '';
    $lastWord = '';

    for($i = 0; $i < $queryLength; $i++) {
      $c = substr($query,$i,1);
      if($string || $field) {
        if($escaped) {
          $escaped = false;
        } else {
          if($string && ($c == "'")) {
            $string = false;
          } elseif($field && ($c == "`")) {
            $field = false;
          } elseif($c == "\\") {
            $escaped = true;
          }
        }
        $result[$lastSection] .= $c;
      } else {
        if($level == $topLevel) {
          if(strpos($identifierChars,$c) !== false) {
            $lastWord .= $c;
          } else {
            $word = strtoupper($lastWord);
            $nextWord = false;           
            if(isset($__roadSqlKeyWords[$word])) {
              $nextWord = $__roadSqlKeyWords[$word];
              if($nextWord === true) {
                if($c == ' ') {
                  $lastWord .= ' ';
                } else {
                  $nextWord = false;
                }
              } else {
                $lastSection = $word;
                if(isset($result[$lastSection])) {
                  return false;
                }
                $result[$lastSection] = '';
              }
            } else {
              $result[$lastSection] .= $lastWord;
            }
            if($nextWord === false) {
              $lastWord = '';
              $result[$lastSection] .= $c;
            }
          }
        } else {
          $result[$lastSection] .= $c;
        }
        if($c == '(') {
          $level ++;
        } elseif($c == ')') {
          if(($topLevel !== null) && ($level == $topLevel)) {
            return false;
          }
          $level --;
        } elseif($c == "'") {
          $string = true;
        } elseif($c == "`") {
          $field = true;
        }        
      }
    }
    if(strlen($result[0]) > 0) {
      return false;
    }
    unset($result[0]);
    $result[$lastSection] .= $lastWord;
    
    foreach(array_keys($result) as $keyWord) {
      $result[$keyWord] = trim($result[$keyWord]);
    }
    return $result;
  }  

  
  
  function sqlImplode($query) {
    global $__roadSqlKeyWords;
    $result = '';
    foreach($__roadSqlKeyWords as $keyWord => $partial) {
      if(!$partial && isset($query[$keyWord])) {
        $result .= ' '.$keyWord.' '.$query[$keyWord];
      }
    }
    return trim($result);
  }
   
  if(!function_exists('http_build_url')) {
    /** @const int replace every part of the first URL when there's one of the second URL */
    define('HTTP_URL_REPLACE',1);
 
    /** @const int join relative paths */
    define('HTTP_URL_JOIN_PATH',2);
 
    /** @const int join query strings */
    define('HTTP_URL_JOIN_QUERY',4);
 
    /** @const int strip any user authentication information */
    define('HTTP_URL_STRIP_USER',8);
 
    /** @const int strip any password authentication information */
    define('HTTP_URL_STRIP_PASS',16);
 
    /** @const int strip any authentication information */
    define('HTTP_URL_STRIP_AUTH',32);
 
    /** @const int strip explicit port numbers */
    define('HTTP_URL_STRIP_PORT',64);
 
    /** @const int strip complete path */
    define('HTTP_URL_STRIP_PATH',128);
 
    /** @const int strip query string */
    define('HTTP_URL_STRIP_QUERY',256);
 
    /** @const int strip any fragments (#identifier) */
    define('HTTP_URL_STRIP_FRAGMENT',512);

    /** @const int strip anything but scheme and host */
    define('HTTP_URL_STRIP_ALL',1024);

    function http_build_url($url, $parts = null, $flags = HTTP_URL_REPLACE) {
      if(is_int($parts)) {
        $flags = $parts;
        $parts = null;
      }
      if(is_string($url)) {
        $url = parse_url($url);
      }
      if(is_string($parts)) {
        $parts = parse_url($parts);
      }
      if(($flags  & HTTP_URL_JOIN_PATH) && (isset($url['path']) || isset($parts['path']))) {
        if(isset($url['path'])) {
          $url['path'] = filePath($url['path']);
          if(isset($parts['path'])) {
            $url['path'] = stripTrailingSlash($url['path']).$parts['path'];
          }                       
        } else {
          $url['path'] = $parts['path'];
        }
        unset($parts['path']);
      }
      if(($flags  & HTTP_URL_JOIN_QUERY) && (isset($url['query']) || isset($parts['query']))) {
        if(isset($url['query']) && isset($parts['query'])) {
          $url['query'] = $url['query'].'&'.$parts['query'];
        } elseif(isset($parts['query'])) {
          $url['query'] = $parts['query'];
        }
        unset($parts['query']);
      }
      if(($flags  & HTTP_URL_JOIN_QUERY) && (isset($url['query']) || isset($parts['query']))) {
        if(isset($url['query']) && isset($parts['query'])) {
          $url['query'] = $url['query'].'&'.$parts['query'];
        } elseif(isset($parts['query'])) {
          $url['query'] = $parts['query'];
        }
        unset($parts['query']);
      }
      if($flags & (HTTP_URL_STRIP_USER | HTTP_URL_STRIP_AUTH | HTTP_URL_STRIP_ALL)) {
        unset($url['user']);
        unset($parts['user']);          
      }
      if($flags & (HTTP_URL_STRIP_PASS | HTTP_URL_STRIP_AUTH | HTTP_URL_STRIP_ALL)) {
        unset($url['pass']);
        unset($parts['pass']);          
      }
      if($flags & (HTTP_URL_STRIP_PORT | HTTP_URL_STRIP_ALL)) {
        unset($url['port']);
        unset($parts['port']);          
      }
      if($flags & (HTTP_URL_STRIP_PATH | HTTP_URL_STRIP_ALL)) {
        unset($url['path']);
        unset($parts['path']);          
      }
      if($flags & (HTTP_URL_STRIP_FRAGMENT | HTTP_URL_STRIP_ALL)) {
        unset($url['path']);
        unset($parts['path']);          
      }
      if(is_array($parts)) {
        $url = array_merge($url,$parts);
      }
      $res = '';
      if(isset($url['scheme'])) {
        $res .= $url['scheme'].'://';
      }
      if(isset($url['user']) && isset($url['pass'])) {
        $res .= urlencode($url['user']).':'.urlencode($url['pass']).'@';
      }        
      if(isset($url['host']) && isset($url['port'])) {
        $res .= $url['host'].':'.$url['port'];
      }
      if(isset($url['host'])) {
        $res .= $url['host'];
      }
      if(isset($url['path'])) {
        $res .= $url['path'];
      }
      if(isset($url['query'])) {
        $res .= '?'.$url['query'];
      }
      if(isset($url['fragment'])) {
        $res .= '#'.$url['fragment'];
      }
      return $res;
    }
  }
  
  define('URL_QUERY_REPLACE',1);
  define('URL_QUERY_MERGE',2);
  define('URL_QUERY_REMOVE',4);
  define('URL_QUERY_ARRAY_REPLACE',8);
  define('URL_QUERY_ARRAY_MERGE',16);
  define('URL_QUERY_MERGE_AND_ARRAY_REPLACE',URL_QUERY_MERGE | URL_QUERY_ARRAY_REPLACE);
  define('URL_QUERY_MERGE_AND_ARRAY_MERGE',URL_QUERY_MERGE | URL_QUERY_ARRAY_MERGE);        
  /**
  * Changes query parameters in url
  *
  * @param mixed Original url as components or string
  * @param mixed Original url as associative array or string
  * @param int Action to be taken  
  * @return void
  */
  function changeUrlQuery($url, $query, $action = URL_QUERY_MERGE_AND_ARRAY_REPLACE)  {
    if(is_string($url)) {
      $url = parse_url($url);
    }
    $xQuery = array();
    if(isset($url['query']) && is_string($url['query'])) {
      parse_str($url['query'],$xQuery);
    }
    if(is_string($query)) {
      parse_str($query,$query);
    }
    if($action & URL_QUERY_REPLACE) {
      $xQuery = $query;
    } elseif($action & URL_QUERY_MERGE) {
      foreach($query as $name => $value) {
        if($value === false) {
          if(isset($xQuery[$name])) {
            unset($xQuery[$name]);
          }
        } elseif(isset($xQuery[$name]) 
          && ($action & URL_QUERY_ARRAY_MERGE)
          && is_array($xQuery[$name])
          && is_array($value)
        ) {
          $xQuery[$name] = array_merge($xQuery[$name],$value);
        } else {
          $xQuery[$name] = $value;
        }
      }
    } elseif($action & URL_QUERY_REMOVE) {
      foreach($query as $name => $value) {
        if(isset($xQuery[$name])) {
          unset($xQuery[$name]);
        }
      }    
    }
    if(count($query) > 0) {
      $url['query'] = http_build_query($xQuery);
      if($url['query'] === '') {
        unset($url['query']);
      }
    }
    return http_build_url($url);
  }
  /**
  * strftime Extension handling encoding conversion
  *
  * @param string Format
  * @param int Timestamp
  * @return string
  */
  function formatDateTime($format, $timestamp = null) {
    if($timestamp === null) {
      $timestamp = time();
    }
    if(defined('CFG_LOCALE_ENCODING')) {
      return iconv(CFG_LOCALE_ENCODING,CFG_ENCODING,strftime($format,$timestamp));
    } else {
      return strftime($format,$timestamp);    
    }    
  }
  function dupeString($string, $count) {
    $res = '';
    for($i = 0; $i < $count; $i++) {
      $res .= $string;
    }
    return $res;
  }
  function intToRoman($value) {
    $res = dupeString('X', floor($value / 10));
    $value = $value % 10;
    if($value == 9) {
      $res .= 'IX';
    } elseif($value >= 5) {
      $res .= 'V' . dupeString('I',$value - 5);
    } elseif($value == 4) {
      $res .= $res . 'IV';
    } else {
      $res .= $res . dupeString('I',$value);
    }
    return $res;
  }  
?>